C$$$  MAIN PROGRAM DOCUMENTATION BLOCK
C     .      .    .                                       .
C     MAIN PROGRAM:  DUMP DATASET'S NUMERIC DATA IN CSV FORMAT.
C     PRGMMR: L. BENJAMIN         ORG: FSL        DATE: 03-30-09
C     
C     ABSTRACT: THIS PROGRAM DUMPS ALL DATASET'S NUMERIC VARIABLES
C      IN CSV FILES.
C     
C     PROGRAM HISTORY LOG:
C     07-07-01  L. BENJAMIN Original version -- MADIS Version TBD
C     14-09-17  L. BENJAMIN Added returned provider code (source_n) from MDSSTA
C     15-02-25  L. BENJAMIN Added header line options for csv output
C     15-04-10  L. BENJAMIN Changed array length to 50 variables
C                           Changed max station to 600000
C                           Reduced MAXDPS to 700000 
C     15-04-29  L. BENJAMIN Changed nvar.gt.MAXPRODS instead hard-coded.
C     16-06-14  L. BENJAMIN Changed max station to 800000
C                           Reduced MAXDPS to 500000 
C     20-03-03  L. BENJAMIN Add acars destination and origin airports
C     20-03-17  L. BENJAMIN No valid lat/lon location, skip line. 
C                           Add check for values exceeding missing vales 
C     20-04-29  L. BENJAMIN No valid elev location, skip line. 
C     22-02-25  L. BENJAMIN V4.5 Much more data and rwis/mobile dataset
C     
C     USAGE:
C     INPUT FILES: (for the entire program)
C     
C     MADIS initialization files in MADIS_STATIC directory   
C     
C     MADIS observation files under MADIS_DATA directories
C     
C     INPUT FILES:
C     
C     csvnumdump.par -- csv input parms file
C
C     OUTPUT FILES:
C     
C     csvnumdump.txt -- csv output file
C
C     standard output  Error messages
C     
C     SUBPROGRAMS CALLED:
C
C     LIBRARY: - MADISLIB:  MINITDS, MSETWIN, MSETDOM, 
C     MSETQC, M_MKFNAM, MTRNTIM, M_FINDWORD,
C     MDSSTA, MGETVCN, MGETDS, MMADISCLOSE
C     
C     REMARKS:
C     
C     1) This program provides a sample in how to use the MADIS library
C     routines as they might be used in an ingest program for an 
C     hourly data assimilation system.  For more details on how to use
C     the various calls and options for each individual dataset,
C     refer to the following tutorial programs:
C     
C     - sfcdump.f               Surface
C     - raobdump.f              Radiosonde
C     - npndump.f               NOAA Profiler Network 
C     - acarsdump.f             Aircraft data (ACARS, AMDARS, etc.)
C     - acarspdump.f            Aircraft data profiles at airports
C     - hydrodump.f             Hydro     
C     - mapdump.f               Multi-Agency Profilers
C     - rdmtrdump.f             Radiometer
C     - satwnddump.f            Satellite winds
C     - satsnddump.f            Satellite soundings
C     - satraddump.f            Satellite radiance
C     - snowdump.f              Snow dataset
C     
C     
C     2) The MADIS library can calculate a number of variables that aren't
C     actually observations stored in the database for each dataset 
C     (e.g., specific humidity isn't a reported quantity, ACARS 
C     pressures aren't reported).  This program demonstrates this by 
C     getting pressure and height for all datasets, and virtual 
C     temperature and specific humidity for all applicable datasets.
C     Fpr more information on what variables are available, see the
C     *variable_list.txt files in the MADIS doc directory.
C     
C     3) See the INSTALL.* files in the MADIS doc directory for 
C     instructions on how to install and build the MADIS library and 
C     programs (INSTALL.unix for Unix/Linux, INSTALL.windows for
C     Windows).
C     
C     4) See the csvnumdump_usage.txt file in the MADIS doc directory for
C     instructions on how to set up a parameter file and run the
C     program.
C    
C     ATTRIBUTES:
C     LANGUAGE: FORTRAN 77 + EXTENSIONS
C     MACHINE: UNIX
C     
C$$$  
C-----------------------------------------------------------------------
C     Declarations notes:
C     
C     Key constants:
C     
C     - 0              successful status return
C     - 999999.0       missing data flag
C     - 800000         maximum number of stations
C     - 1600000        maximum number of variables data points 
C     - 20             maximum number of variables per row (if number of datasets is 1 then allow 50) 
C
C     2) Station arrays should be dimensioned to 800000, for the maximum 
C        number of station records in any dataset.
C
C     3) Data arrays should be dimensioned to 1600000, for the maximum 
C        number of station records in any dataset.
C
C     4) If the maximum number of variables per row increases above 12
C        increase the write 2000 format statement.
C
      implicit none


      integer success_p,nhr,ilevtype
      real miss_p
      parameter (success_p = 0)
      parameter (miss_p = 999999.0)

      real out_miss
      parameter (out_miss = -99999.000000)

      integer maxstn
c$$$      parameter (maxstn = 200000)
c$$$      parameter (maxstn = 600000)
      parameter (maxstn = 800000)

      integer istatus,i,line,idb,j,k,lnnum,ii,kk,l,jj
      integer nsta,nobs,nstamax
      character*9 atime
      character*6 db
      character*10 stanam(maxstn)
      character*9 timeob(maxstn)
      character*10 provdr(maxstn)
      character*95 csvstr
      character*(2000) csvstrheader
      integer csvstrendlenheader
      integer j2

      character*8 subpvdr
      character*5 hmstr
      character*10 dstr
      character*256 cln
      character*1 cvsheadercnt(9)

      INTEGER*4 MAXPRODS
      PARAMETER(MAXPRODS = 50)

      INTEGER*4 MAXDPS
c$$$      PARAMETER(MAXDPS = 1600000)
c$$$      PARAMETER(MAXDPS = 700000)
      PARAMETER(MAXDPS = 490000)
      INTEGER*4 lastnobs

      character*40 header,tmpline
      integer ilatlon
      integer m_lstchr,firstdataset,lastdataset,ids
      integer itextxml,vce,imissval,idx,totidx
      integer itextxml_ext


      CHARACTER*13 CHARRDATA1(MAXDPS,MAXPRODS)

      REAL RDATA1(MAXDPS,MAXPRODS)
      INTEGER QCA1(MAXDPS,MAXPRODS),QCR1(MAXDPS,MAXPRODS)
      CHARACTER*1 QCD1(MAXDPS,MAXPRODS)


      REAL RDATA1HT(MAXDPS,MAXPRODS)
      REAL DATAHT
      CHARACTER*8 CDATAHT
      INTEGER QCA1HT(MAXDPS,MAXPRODS),QCR1HT(MAXDPS,MAXPRODS)
      CHARACTER*1 QCD1HT(MAXDPS,MAXPRODS)


      real lat(maxstn),lon(maxstn),elev(maxstn)
      integer source_n(maxstn)
      integer wmoid(maxstn)
      integer nlevels(maxstn)
      integer nlevelsht(maxstn)
      integer nl
      

      integer gttotnvar
      integer gtnvar(1000)
      integer gtlnnum
      character*7 gtvcode(MAXPRODS,1000)
      character*13 hdvcode

      character*300 vline
      integer lstchr

      integer vcn,profile_type(MAXPRODS)
      character*7 vcode(MAXPRODS)
      character*7 vcode_ds(MAXPRODS)
      character*21 vcstr(MAXPRODS)
      character*1 vtype(MAXPRODS),c,def_vtype      
      integer minbck,minfwd,recwin,qctyp,ipvd,idmtyp,tform,ctime
      integer idatasets,satwis_recwin
      real dummy,ri(maxstn),rj(maxstn)
      integer idummy,time,def_profile_type
      integer nvar,totnvar
      real rpar(7)
      integer ipar(2)
      character*13 tstr
      integer maxproviders
      parameter(maxproviders=50)
      integer pvdrnamsize
      parameter(pvdrnamsize=10)

      integer MXACARS_L
      parameter(MXACARS_L=60000)

      character*3 alt_vcode,lat_vcode,lon_vcode
      integer lat_qca(MXACARS_L),lon_qca(MXACARS_L),alt_qca(MXACARS_L)
      integer lat_qcr(MXACARS_L),lon_qcr(MXACARS_L),alt_qcr(MXACARS_L)
      character*1 lat_qcd(MXACARS_L),lon_qcd(MXACARS_L)
      character*1 alt_qcd(MXACARS_L)
      character*7 oairport_vcode,dairport_vcode

      character*(pvdrnamsize) pvdrnam(maxproviders)
      character*63 twstr(0:4)
      integer twstrlen(0:4)
      character*16 qcstr(0:1)
      logical isprofile,validcode(50)
      integer numds,ds_i,datasetid(14)
      character*7 datasetnm(14)

      CHARACTER*6 ORIGAIRPORT(maxstn)
      CHARACTER*6 DESTAIRPORT(maxstn)
      logical ORIGAPT
      logical DESTAPT
      integer ORIGAPTIDX
      integer DESTAPTIDX
      integer ich
      integer cnt_chars


C----------------------------------------------------------------------
C     datasetid index is MGENVCN's CD value +1
C     NPN_DS_P = 2, datasetid value is  2+1 = 3
C     datasetid index is also the value entered in the par file
C----------------------------------------------------------------------
      data datasetid /1,2,3,4,5,6,7,8,9,10,11,12,13,14/
      data datasetnm /
     1     'SFC','RAOB','NPN','ACARS','MAP','RDMTR',
     2     'ACARSP','HYDRO','SATWND',' ','SATSND','SATRAD',
     3     'SNOW','WISDOM'/ 
     
      data twstr/
     1     'Return all records within the file containing nominal time',  
     2     'N/A','N/A','N/A',
     3     'Return all records in window'/
      data twstrlen/58,3,3,3,28/
      data qcstr/'No Filter','Coarse'/

C      acars codes
      data alt_vcode/'HT'/
      data lat_vcode/'LAT'/
      data lon_vcode/'LON'/

      data oairport_vcode/'ORIGAPT'/
      data dairport_vcode/'DESTAPT'/


      cvsheadercnt(1)='1'
      cvsheadercnt(2)='2'
      cvsheadercnt(3)='3'
      cvsheadercnt(4)='4'
      cvsheadercnt(5)='5'
      cvsheadercnt(6)='6'
      cvsheadercnt(7)='7'
      cvsheadercnt(8)='8'
      cvsheadercnt(9)='9'


      ORIGAPT = .FALSE.
      DESTAPT = .FALSE.
      ORIGAPTIDX = 0
      DESTAPTIDX = 0      
C----------------------------------------------------------------------
C     PROGRAM SETUP
C     -------------
C     
C     Open an output file for this program.

      open(unit=1,file='csvnumdump.txt',status='UNKNOWN')

C     Open the parameter file that will specify the user's selections.

      open(unit=2,file='csvnumdump.par',status='OLD')
      
C----------------------------------------------------------------------
C     MANDATORY MADIS INITIALIZATION
C     -----------------------------
C     
C     Mandatory initialization which must be done before
C     any other MADIS library calls.
C     
C     
C     
C     Initialize the MADIS satellite wind subsystem
C     ---------------------------------------------
C     
C     The user selects the "subsystem" to be initialized by
C     specifying the following arguments to the MINIT call.
C     This is done by the MINITDS call which handles the 
C     following subsystems:
C
C     1) Name of the subsystem:
C     
C     'SFC'        Surface                 
C     'RAOB'       Radiosonde
C     'NPN'        NOAA Profiler Network
C     'ACARS'      Aircraft data (ACARS, AMDARS, etc.)
C     'ACARSP'     Aircraft data profiles at airports
C     'MAP'        Multi-Agency Profilers
C     'HYDRO'      Hydro
C     'RDMTR'      Radiometer
C     'SATWND'     Satellite wind
C     'SATSND'     Satellite sounding      (used here)
C     'SATRAD'     Satellite radiance
C     'SNOW'       Snow
C     
C     2) Name of the database:
C     
C     'FSL'        Data obtained from FSL
C     'AWIPS'      Data obtained from AWIPS
C     
C     Note that the list of variables with QC found in the MADIS
C     documentation applies to the FSL database.  For the AWIPS
C     database, the only QC is on experimental GOES winds obtained 
C     from FSL.  Otherwise, the QC information returned by API calls
C     will indicate that no QC has been applied.
C     
C     3) Error handling (.true./.false.):
C     
C     If ".true.", then error messages will be written to standard
C     output.  If ".false.", no error messages will be output (but
C     ISTATUS will still contain an error code).  The error codes
C     can be found in errors.txt in the doc directory.

C     PAR FILE INPUT:  Database ('FSL' or 'AWIPS')

C     The necessary inputs are found in the parameter file, along with
C     a number of lines of explanation, section delimiters, etc. meant
C     to aid readability.  Throughout this program, we'll be skipping
C     non-input lines, reading necessary inputs, and checking for I/O
C     errors.  Otherwise, the validity of most of user's choices will
C     be tested by the affected MADIS library routines.  Invalid 
C     choices will be reported to standard output, and in most cases,
C     the program will then stop.

C-----------------------------------------------------------------------
C     MANDATORY MADIS INITIALIZATION
C     -----------------------------

C     Initialize the MADIS subsystems to be used, using the FSL
C     database, and with error messages echoed to standard output.

C FSL
      idb = 0
      lastnobs = MAXDPS

      CALL MINITDS(istatus)
      if(istatus.ne.success_p)stop
      

      line = 0
      do i = 1, 3
         line = line + 1
         read(2,*,err=825,end=850)
      enddo
      line = line + 1
      read(2,145,err=825,end=850)db
 145  format(a)

      write(*,500)'MADIS CSVNUMDUMP Activity Log'
 500  format(1x,a)
      write(*,500)'-----------------------------'

      write(*,501)'Database'
 501  format(/,1x,a)
      write(*,401)db
 401  format(3x,a)
 601  format(3x,'Only ',i5,'providers selections allowed.')



CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C-----------------------------------------------------------------------
C OPTIONAL MADIS SETUP CALLS
C -------------------------
C
C     Optional calls to limit which obs MADIS will return.
C     These can be selected at any time in the code,
C     and will then take effect upon the next call to
C     MDSSTA (see below).  The default behavior (what
C     happens if you don't call the routine) is described
C     below.
C
C     Select time window
C     ------------------
C
C     The user selects the nominal time of the data to be returned
C     when using MADIS routines to read obs and station information.
C     Time "windowing" is used so that the MADIS routines will
C     know how close in time each station's observation time needs
C     to be to the nominal time in order to be included, and whether
C     or not duplicate records from the same station that are within
C     the time window should be included.  The time window is
C     specified with the MSETWIN subroutine, which allows the user
C     to select the number of minutes before and after the nominal
C     time to be used for the window, and how duplicates should
C     be handled.  For more details, see msetwin.txt in the doc 
C     directory.
C
C     If MSETWIN is not called, the default will be to return all
C     records (including multiple reports from the same station)
C     in the file containing the nominal time.  For non-mesonet
C     stations, this will return observation times between 15
C     minutes before the hour and 44 minutes after the hour.  For
C     mesonet stations, data will cover from the hour until 59
C     minutes after the hour.
C
C     Surface stations report observation times around the hour, and 
C     many stations report multiple times per hour, so different
C     time window specifications can produce very different sets
C     of data.
C
C     The following sample MSETWIN call will return only one record 
C     per station within the specified time window (-30 minutes -->
C     nominal time --> +30 minutes).  If a station has multiple reports
C     within the time window, only the latest one within the window
C     will be returned (RECWIN = 3).
C
C     Sample MSETWIN call:
C
C     minbck = -30
C     minfwd = 30
C     recwin = 3
C     CALL MSETWIN(minbck,minfwd,recwin,istatus)

C     PAR FILE INPUT:  Time window parameters (minbck, minfwd, recwin)
C                      [set all to 0 for default]
      do i = 1, 3
         line = line + 1
         read(2,*,err=825,end=850)
      enddo
      line = line + 1
      read(2,*,err=825,end=850)minbck
      line = line + 1
      read(2,*,err=825,end=850)minfwd
      line = line + 1
      read(2,*,err=825,end=850)recwin

      write(*,501)'Time Window'

      if(recwin.ge.0.and.recwin.le.4)then
         satwis_recwin = recwin

         write(*,401)twstr(recwin)(1:twstrlen(recwin))

         if(minbck.ne.0.or.minfwd.ne.0.or.recwin.ne.0)then

            write(*,502)'Start minute relative to nominal time ',minbck
 502        format(3x,a,i4)
            write(*,502)'End minute relative to nominal time   ',minfwd

            CALL MSETWIN(minbck,minfwd,recwin,istatus)
            if(istatus.ne.success_p)stop
            satwis_recwin = 4

         endif

      else

         write(*,500)'CSVNUMDUMP: INVALID WINDOW RECORD TYPE'
         stop

      endif


CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

c$$$C     PAR FILE INPUT:  Time window parameters (minbck, minfwd, recwin)
c$$$C     [set all to 0 for default]
c$$$      do i = 1, 3
c$$$         line = line + 1
c$$$         read(2,*,err=825,end=850)
c$$$      enddo
c$$$      line = line + 1
c$$$C     Currently we are hardwired to only one hour of data.
c$$$      read(2,*,err=825,end=850)nhr
c$$$      line = line + 1
c$$$      minbck = -59
c$$$      minfwd = 0
c$$$      read(2,*,err=825,end=850)recwin
c$$$
c$$$      write(*,501)'Time Window'
c$$$
c$$$C     Set record window processing filter
c$$$      if(recwin.ge.0.or.recwin.eq.4)then
c$$$
c$$$         write(*,401)twstr(recwin)(1:twstrlen(recwin))
c$$$
c$$$         if(minbck.ne.0.or.minfwd.ne.0.or.recwin.ne.0)then
c$$$
c$$$ 502        format(3x,a,i4)
c$$$
c$$$            CALL MSETWIN(minbck,minfwd,recwin,istatus)
c$$$            if(istatus.ne.success_p)stop
c$$$
c$$$         endif
c$$$
c$$$      else
c$$$
c$$$         write(*,500)'CSVNUMDUMP: INVALID WINDOW RECORD TYPE'
c$$$         stop
c$$$
c$$$      endif
      do i = 1, 7
         line = line + 1
         read(2,*,err=825,end=850)
      enddo
      line = line + 1
      read(2,*,err=825,end=850)ipvd
C Skip comments lines
      line = line + 1
      read(2,*,err=825,end=850)
      line = line + 1
      read(2,*,err=825,end=850)

      write(*,501)'Providers'

      if(ipvd.gt.0)then

         do i = 1, ipvd
 
            line = line + 1
            if(i.le.maxproviders)then
               read(2,145,err=825,end=850)pvdrnam(i)
            else
               read(2,*,err=825,end=850)
            endif
         enddo
         if(ipvd.gt.maxproviders)then
            write(*,601)maxproviders
            ipvd = maxproviders
         endif
      else if(ipvd.eq.0)then

         write(*,401)'ALL-PROVIDERS'

      else

         write(*,500)'CSVNUMDUMP: INVALID PROVIDER SELECTION'
         stop

      endif



C     Select a subset of datasets
C     ------------------------------------------
C     
C     PAR FILE INPUT:  Data Sets (0 - all providers, 
C     N - N lines of provider names follow)

      do i = 1, 3
         line = line + 1
         read(2,*,err=825,end=850)
      enddo
      line = line + 1
      read(2,*,err=825,end=850)idatasets
      line = line + 1
      read(2,*,err=825,end=850)
      line = line + 1
      read(2,*,err=825,end=850)
      line = line + 1
      read(2,*,err=825,end=850)
      line = line + 1
      read(2,*,err=825,end=850)

      write(*,501)'Data Sets'
      if(idatasets.gt.0)then
         numds=idatasets
         do i = 1, idatasets
            line = line + 1
            read(2,*,err=825,end=850)datasetid(i)
            datasetid(i) = datasetid(i) + 1
            if(istatus.ne.success_p)stop
            write(*,401)datasetnm(datasetid(i))
         enddo

      else if(idatasets.eq.0)then
         numds=14
         istatus = success_p
         write(*,401)'ALL-DATA SETS'

      else

         write(*,500)'INVALID DATA SET SELECTION'
         stop

      endif

C     Select a horizontal domain
C     --------------------------
C     
C     The horizontal domain can be limited to match one of two map
C     projections (polar stereographic, Lambert conformal conic) or it
C     can be limited to a latitude/longitude rectangle.  Only stations
C     within the domain will be returned.  If one of the map  
C     projections is selected, any winds (of any form) returned by 
C     subsequent MGETSDS???-TBD calls will be rotated to match the 
C     projection.  See msetdom.txt in the doc directory for an 
C     explanation of the parameters needed to define the domains.
C     
C     If MSETDOM is not called, all available observations will be
C     returned, regardless of their location.
C     
C     The example given here is for the CONUS 212 Lambert conformal 
C     conic projection.  
C     
C     CALL MSETDOM(3,40635.25,12.19,-133.459,-95.0,25.0,25.0,dummy,
C     1             185,129,istatus)

C     PAR FILE INPUT:  Domain Filter (0 - don't filter, 
C     1 - lat/lon corners,
C     2 - Polar Stereographic grid,
C     3 - Lambert Conformal Conic grid)

      do i = 1, 3
         line = line + 1
         read(2,*,err=825,end=850)
      enddo

      line = line + 1
      read(2,*,err=825,end=850)idmtyp

C     We'll read in the parameters appropriate for the selected
C     domain filter, and skip over all of the other lines in the
C     rest of this section of the parameter file.

      write(*,501)'Domain Filter'

      if(idmtyp.eq.1)then

         do i = 1, 6
            line = line + 1
            read(2,*,err=825,end=850)
         enddo
         do i = 1, 4
            line = line + 1
            read(2,*,err=825,end=850)rpar(i)
         enddo
         do i = 1, 23
            line = line + 1
            read(2,*,err=825,end=850)
         enddo

         write(*,401)'Latitude/Longitude Corners'
         write(*,402)'SW corner latitude (north)                 ',
     1        rpar(1)
 402     format(3x,a,f13.6)
         write(*,402)'SW corner longitude (east)                 ',
     1        rpar(2)
         write(*,402)'NE corner latitude (north)                 ',
     1        rpar(3)
         write(*,402)'NE corner longitude (east)                 ',
     1        rpar(4)

         CALL MSETDOM(idmtyp,rpar(1),rpar(2),rpar(3),rpar(4),dummy,
     1        dummy,dummy,idummy,idummy,istatus)

      else if(idmtyp.eq.2)then

         do i = 1, 13
            line = line + 1
            read(2,*,err=825,end=850)
         enddo
         do i = 1, 7
            line = line + 1
            read(2,*,err=825,end=850)rpar(i)
         enddo
         do i = 1, 2
            line = line + 1
            read(2,*,err=825,end=850)ipar(i)
         enddo
         do i = 1, 11
            line = line + 1
            read(2,*,err=825,end=850)
         enddo

         write(*,401)'Polar Stereographic Grid'
         write(*,402)'Grid box size (meters)                     ',
     1        rpar(1)
         write(*,402)'Latitude (north) of 1st grid point         ',
     1        rpar(2)
         write(*,402)'Longitude (east) of 1st grid point         ',
     1        rpar(3)
         write(*,402)'Orientation longitude (east)               ',
     1        rpar(4)
         write(*,402)'I-coordinate of pole                       ',
     1        rpar(5)
         write(*,402)'J-coordinate of pole                       ',
     1        rpar(6)
         write(*,402)'Latitude (north) at which X-Y scale is true',
     1        rpar(7)
         write(*,403)'Number of grid points (X)                  ',
     1        ipar(1)
         write(*,403)'Number of grid points (Y)                  ',
     1        ipar(2)
 403     format(3x,a,i6)   

         CALL MSETDOM(idmtyp,rpar(1),rpar(2),rpar(3),rpar(4),rpar(5),
     1        rpar(6),rpar(7),ipar(1),ipar(2),istatus)

      else if(idmtyp.eq.3)then

         do i = 1, 25
            line = line + 1
            read(2,*,err=825,end=850)
         enddo
         do i = 1, 6
            line = line + 1
            read(2,*,err=825,end=850)rpar(i)
         enddo
         do i = 1, 2
            line = line + 1
            read(2,*,err=825,end=850)ipar(i)
         enddo

         write(*,401)'Lambert Conformal Conic Grid'
         write(*,402)'Grid box size (meters)                     ',
     1        rpar(1)
         write(*,402)'Latitude (north) of 1st grid point         ',
     1        rpar(2)
         write(*,402)'Longitude (east) of 1st grid point         ',
     1        rpar(3)
         write(*,402)'Orientation longitude (east)               ',
     1        rpar(4)
         write(*,402)'Latitude (north) of 1st tangent            ',
     1        rpar(5)
         write(*,402)'Latitude (north) of 2nd tangent            ',
     1        rpar(6)
         write(*,403)'Number of grid points (X)                  ',
     1        ipar(1)
         write(*,403)'Number of grid points (Y)                  ',
     1        ipar(2)

         CALL MSETDOM(idmtyp,rpar(1),rpar(2),rpar(3),rpar(4),rpar(5),
     1        rpar(6),dummy,ipar(1),ipar(2),istatus)

      else if(idmtyp.eq.0)then

         write(*,401)'No Filter'

         do i = 1, 33
            line = line + 1
            read(2,*,err=825,end=850)
         enddo
         
      else

         write(*,500)'CSVNUMDUMP: INVALID DOMAIN TYPE'
         stop

      endif
      if(istatus.ne.success_p)then
         write(*,500)'CSVNUMDUMP: ERROR IN DOMAIN FILTER'
         stop
      endif
C     Select QC level of data to be returned
C     --------------------------------------
C     
C     The primary variables in all MADIS datasets have been screened
C     by automated quality control (QC) checks.  Different variables
C     have different levels of QC, or "stages", applied to them.
C     In addition to the QC processing done by the satellite provider
C     (who generally toss observations that are below their own quality
C     thresholds), currently the satellite winds have a MADIS level 1 
C     validity check applied to them.  See TDB-satsnd_qc_variable_list.txt 
C     in the doc directory for more information.
C     
C     Each call to MDSSTA will return obs and associated QC
C     information.  If the user doesn't want to interpret the QC info,
C     he or she can limit the returned obs to selectable levels of
C     quality.  In that case, obs that don't meet the selected QC level
C     will be replaced with missing data flags, and the QC information
C     itself doesn't need to be examined by the calling program.
C     
C     If MSETQC is not called, all obs will be returned (along with QC
C     info).
C     
C     There are three ways that the user can request and process the QC
C     information:
C     
C     1) Don't call MSETQC at all, and examine the QC info before
C     deciding whether or not to use each observation.  This
C     allows the calling program the maximum amount of control
C     over what's available.
C     
C     2) If you know ahead of time that the same level of QC is
C     available for all variables you'll be using, call MSETQC
C     one time to select the desired QC level.  Note that the
C     level selected by MSETQC will stay in effect until the
C     next time MSETQC is called.
C     
C     3) To require that all variables pass the highest level
C     of QC that are specified for them, call MSETQC one time
C     and select QC level 99.
C     
C     User's of MADIS data who want the data filtered by QC results,
C     but don't want to interpret the QC information themselves, should
C     call MSETQC one time and select QC level 99.

C     PAR FILE INPUT:  QC level (0-none, 1-coarse, 99-highest possible)
      do i = 1, 3
         line = line + 1
         read(2,*,err=825,end=850)
      enddo
      line = line + 1
      read(2,*,err=825,end=850)qctyp

      write(*,501)'QC Filter'
      if(qctyp.ge.0.and.qctyp.le.1)then
         write(*,401)qcstr(qctyp)
      else if(qctyp.eq.99)then
         write(*,401)'Highest Possible'
      else
         write(*,500)'INVALID QC TYPE'
         stop
      endif
C     Select the QC level to be returned.

      CALL MSETQC(qctyp,istatus)
      if(istatus.ne.success_p)then
         write(*,500)'CSVNUMDUMP: ERROR SETTING QC LEVEL'
         stop
      endif
C----------------------------------------------------------------------
C     SELECT NOMINAL TIME OF DATA TO BE RETURNED 
C     ------------------------------------------
C     
C     The MADIS library's native time zone and format is Universal 
C     Coordinated Time, using a Julian date, in the form of "YYJJJHHMM"
C     , where:
C     
C     YY  = 2-digit year (good from 1980 - 2179)
C     JJJ = Julian date of the year (001-366)
C     HH  = Hour (00-23)
C     MM  = Minute (00-59)
C     
C     MADIS also provides support for a "Month/Day" date format, using
C     "YYYYMMDD_HHMM", where:
C     
C     YYYY = 4-digit year (still only good from 1980 - 2179)
C     MMDD = Month/Day 
C     HHMM = Hour/Minute 
C     
C     The MADIS library includes a basic set of routines that the 
C     caller can use to:
C     
C     - Add & subtract time in the native format.  (MCHGTIM)
C     - Translate the native format to integers representing year, 
C     month, day, hour, and minute.  (MINTTIM)
C     - Translate integers representing year, month, day, hour and
C     minute into the native format.  (MCHRTIM)
C     - Translate between Julian and Month/Day formats (MTRNTIM)
C     
C     See time_routines.txt in the doc directory for details on how 
C     to use these calls.
C     
C     Also note that nominal times can be specified at any minute, not
C     just at the top of the hour (minute 00).

C     PAR FILE INPUT:  Time format to use (0 - Julian, 1 - Month/Day)
      
      write(*,501)'Nominal Time'

      do i = 1, 7
         line = line + 1
         read(2,*,err=825,end=850)
      enddo
      line = line + 1
      read(2,*,err=825,end=850)tform
      if(tform.ne.0.and.tform.ne.1)then
         write(*,500)'CSVNUMDUMP: INVALID TIME TRANSLATION FORMAT'
         stop
      endif

C     PAR FILE INPUT:  Nominal time (0 - specified in par file,
C     1 - use current time)

      line = line + 1
      read(2,*,err=825,end=850)
      line = line + 1
      read(2,*,err=825,end=850)ctime
      line = line + 1
      read(2,*,err=825,end=850)

C     PAR FILE INPUT:  Nominal time (in selected format)

      line = line + 1
      read(2,144,err=825,end=850)tstr
 144  format(a)

C     If the user wants to use the current time, get it and translate 
C     that into the chosen format (via our internal formats).  
C     Otherwise we have the chosen format in TSTR right now.

      if(ctime.eq.1)then
         i = TIME()
         i = i - 1325376000
         CALL M_MKFNAM(i,atime,istatus)
         if(tform.eq.1)then
            CALL MTRNTIM(atime,2,tstr,istatus)
         else
            tstr = atime
         endif
      endif


      if(tform.eq.1)then
         CALL MTRNTIM(tstr,1,atime,istatus)
         if(istatus.ne.success_p)stop
      else
         atime = tstr
      endif

      write(*,401)tstr

      line = line + 1
      read(2,*,err=825,end=850)

      line = line + 1
      read(2,144,err=825,end=850)header

C     The latitude and longitude can either be output using two digits
C     after the decimal point (the original setting prior to adding 
C     this option) or the full 7-digits of possible precision can be 
C     selected.  This means latitude will have 5 digits after the
C     decimal point and longitude 4 digits.
C     
C     PAR FILE INPUT:  Latitude/longitude format

      line = line + 1
      read(2,*,err=825,end=850)

      line = line + 1
      read(2,*,err=825,end=850)ilatlon
      if(ilatlon.eq.0)then
         write(*,501)'Lat/lon with 2 digits after decimal place'
      else if(ilatlon.eq.1)then
         write(*,501)'Lat with 5 digits after decimal place, lon',
     1        ' with 4 digits'
      else
         write(*,500)'CSVNUMDUMP: INVALID LAT/LON OUTPUT TYPE'
         stop
      endif

C Spacers
      do i = 1, 2
         line = line + 1
         read(2,145,err=825,end=850)tmpline
      enddo


C  We are starting at 2 for historical reasons
      line = line + 1
      read(2,*,err=825,end=850)itextxml
      if(itextxml.eq.2)then
         write(*,501)'Comma-separated-variable (CSV) '//
     1        'text output without QC info'
      else if(itextxml.eq.3)then
         write(*,501)'CSV text output with QC data descriptors'
      else if(itextxml.eq.4)then
         write(*,501)'CSV text output with full QC information'
      else if(itextxml.eq.5)then
         write(*,501)'CSV text output with headers and '//
     1        'QC data descriptors'
         itextxml_ext = 5
         itextxml = 3
      else if(itextxml.eq.6)then
         write(*,501)'CSV text output with headers and '//
     1        'full QC information'
         itextxml_ext = 6
         itextxml = 4
      else
         write(*,500)'CSVNUMDUMP: INVALID TEXT/XML OUTPUT FORMAT'
         stop
      endif

C     Options comments
      line = line + 1
      read(2,*,err=825,end=850)
      line = line + 1
      read(2,*,err=825,end=850)

C     Check for extended options lines (header info)
      line = line + 1
      read(2,145,err=825,end=850)tmpline
C Check for Extended CSV version
      j2 = index(tmpline,'5 - Extended')
      if(j2.gt.0)then
         line = line + 1
         read(2,*,err=825,end=850)
         line = line + 1
         read(2,*,err=825,end=850)
      endif

      line = line + 1
      read(2,*,err=825,end=850)imissval
      if(itextxml.ge.2.and.itextxml.le.4.)then
         if(imissval.eq.0)then
            write(*,501)'Using -99999 for missing '//
     1           'fields'
         else if(imissval.eq.1)then
            write(*,501)'Using blanks for missing fields'
            imissval = 1
         else
            write(*,500)'CSVNUMDUMP: INVALID CSV MISSING '//
     1           'FIELD VALUE'
            stop
         endif
      endif

C     Skip to levels  identifier

      do i = 1, 2
         line = line + 1
         read(2,*,err=825,end=850)
      enddo

C     Skip to variables
      do i = 1, 4
         line = line + 1
         read(2,*,err=825,end=850)
      enddo


CCCCCCCCCC
C-----------------------------------------------------------------------
C     PROGRAM SETUP
C     -------------
C     

C----------------------------------------------------------------------
C     LOOP THROUGH DESIRED VARIABLES:
C     -------------------------------
C     
C     Variables are specified by code names.  The list of code names
C     can be found in the first column of TBD-satsnd_variable_list.txt in 
C     the doc directory.
C     
C     This program will read in one variable code per line in the
C     parameter file, read it, output it, then move on to the next
C     variable until the end of the parameter file is reached.

      write(*,501)'Variables'
C     PAR FILE INPUT:  1 to 6 variable codes (free-formatted)

      gtlnnum = 0
      gttotnvar = 0
 100  read(2,145,err=955,end=955)vline
      lstchr = M_LSTCHR(vline,300)

      gtlnnum = gtlnnum + 1
C     Parse the line to determine the number of variables and their
C     code names (1 to MAXPRODS code names separated by spaces).

      nvar = 0
      totnvar = 0
      l =-1
      j = -1
      do i =  1, MAXPRODS
         CALL M_FINDWORD(vline,lstchr,j,k,l)
         j = l
         if(k.eq.0)then
            go to 301
         else
            nvar = nvar + 1
            totnvar = totnvar + 1
            gtvcode(nvar,gtlnnum) = vline(k:l)
            if(gtvcode(nvar,gtlnnum).eq.'SKYCVLB')then
               totnvar = totnvar + 5
            else if(gtvcode(nvar,gtlnnum).eq.'PCPINT')then
               totnvar = totnvar + 1
            else if(gtvcode(nvar,gtlnnum).eq.'PCPTYPE')then
               totnvar = totnvar + 1
            else if(gtvcode(nvar,gtlnnum).eq.'ORIGAPT')then
               ORIGAPT = .TRUE.
               ORIGAPTIDX = nvar
            else if(gtvcode(nvar,gtlnnum).eq.'DESTAPT')then
               DESTAPT = .TRUE.
               DESTAPTIDX = nvar
            endif
            if(totnvar.gt.MAXPRODS)then
               nvar = nvar -1
               write(*,501)'Max Variables surpassed, truncating after '
     1              //gtvcode(nvar,gtlnnum)
               goto 301
            endif
         endif
         gtnvar(gtlnnum) = nvar
      enddo
 301  continue
      if(nvar.eq.0)then
         write(*,*)'No var on line ',gtlnnum, ' skipping line.' 
         go to 100
      endif
      gttotnvar = gttotnvar + nvar

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C csvstrheadder
      csvstrheader = ''
      csvstrendlenheader = 0
      if(itextxml.eq.3 .or. itextxml.eq.4)then
         write(csvstrheader,6002)gtlnnum
         csvstrendlenheader = 109
         do i = 1, nvar
            vcode(i) = gtvcode(i,gtlnnum)
c$$$            validcode(i) = .true.
c$$$            CALL MGETVCN(vcode(i),vcn,vtype(i),istatus)
            hdvcode = vcode(i)
            if(vcode(i).ne.'SKYCVLB'.and.
     1           vcode(i).ne.'PCPINT'.and.vcode(i).ne.'PCPTYPE')then
C VAR
               idx = csvstrendlenheader+1
               csvstrheader(idx:idx+12) = hdvcode
               csvstrendlenheader = idx+12
               idx = csvstrendlenheader+1
               csvstrheader(idx:idx) = ','
C QC
               csvstrendlenheader = idx
               idx = csvstrendlenheader+1
               if(itextxml.eq.3)then
                  csvstrheader(idx:idx+1) = 'D,'
                  csvstrendlenheader = idx +1
               else if(itextxml.eq.4)then
                  csvstrheader(idx:idx+11) = 'D,QCA ,QCR ,'
                  csvstrendlenheader = idx +11
               endif
            else if(vcode(i).eq.'SKYCVLB')then
               do j=1,6
C VAR
                  idx = csvstrendlenheader+1
                  hdvcode = 'SKYCVLB_'//cvsheadercnt(j)
                  csvstrheader(idx:idx+12) = hdvcode
                  csvstrendlenheader = idx+12
                  idx = csvstrendlenheader+1
                  csvstrheader(idx:idx) = ','
C QC
                  csvstrendlenheader = idx
                  idx = csvstrendlenheader+1
                  if(itextxml.eq.3)then
                     csvstrheader(idx:idx+1) = 'D,'
                     csvstrendlenheader = idx +1
                  else if(itextxml.eq.4)then
                     csvstrheader(idx:idx+11) = 'D,QCA ,QCR ,'
                     csvstrendlenheader = idx +11
                  endif
               enddo
            else if(vcode(i).eq.'PCPINT')then
               do j=1,2
C VAR
                  idx = csvstrendlenheader+1
                  hdvcode = 'PCPINT_'//cvsheadercnt(j)
                  csvstrheader(idx:idx+12) = hdvcode
                  csvstrendlenheader = idx+12
                  idx = csvstrendlenheader+1
                  csvstrheader(idx:idx) = ','
C QC
                  csvstrendlenheader = idx
                  idx = csvstrendlenheader+1
                  if(itextxml.eq.3)then
                     csvstrheader(idx:idx+1) = 'D,'
                     csvstrendlenheader = idx +1
                  else if(itextxml.eq.4)then
                     csvstrheader(idx:idx+11) = 'D,QCA ,QCR ,'
                     csvstrendlenheader = idx +11
                  endif
               enddo
            else if(vcode(i).eq.'PCPTYPE')then
               do j=1,2
C VAR
                  idx = csvstrendlenheader+1
                  hdvcode = 'PCPTYPE_'//cvsheadercnt(j)
                  csvstrheader(idx:idx+12) = hdvcode
                  csvstrendlenheader = idx+12
                  idx = csvstrendlenheader+1
                  csvstrheader(idx:idx) = ','
C QC
                  csvstrendlenheader = idx
                  idx = csvstrendlenheader+1
                  if(itextxml.eq.3)then
                     csvstrheader(idx:idx+1) = 'D,'
                     csvstrendlenheader = idx +1
                  else if(itextxml.eq.4)then
                     csvstrheader(idx:idx+11) = 'D,QCA ,QCR ,'
                     csvstrendlenheader = idx +11
                  endif
               enddo
            endif
         enddo
      endif

 6002 format(i2,',ELEV    ,LAT      ,LON        ,DATE       ,TIME,',
     1     'SITEID     ,PROVIDER  ,SUBPVDR,DATASET      ,OBSELEV ,',
     2     'PTY,')
 168  format(a)
      if(itextxml_ext.gt.0)then
         write(1,168)csvstrheader(1:csvstrendlenheader)
      endif

      go to 100
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
 955  continue
      if(gttotnvar .eq.0)then
         write(*,*)'No variables found.  Exiting...'
         goto 900
      endif
      lnnum = 0
 110  continue
      lnnum = lnnum + 1
      if(lnnum .gt. gtlnnum)goto 875
      nvar = gtnvar(lnnum)
      do i = 1,gtnvar(lnnum)
c$$$         nvar = nvar + 1
         vcode(i) = gtvcode(i,lnnum)
      enddo

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC



      if(nvar.eq.0) then
         write(*,*)'0 nvar, read next ',nvar,lnnum
         go to 110
      endif
C     Use internal MADIS routines (MGETVCN, MGENVCN) to translate the 
C     variable code name into the MADIS variable code number, and to
C     determine the type (numeric or character) of the variable.  The
C     code number is also then used to determine the profile type.  
C     This is needed here because this program is an all-purpose utility
C     meant to handle any desired profile type.  In general, a user's 
C     MADIS program shouldn't have to determine the profile type (as you 
C     would know what variables you're reading).


      firstdataset = 1
      lastdataset = 11



      do 103 ids = 1,numds
         ds_i = datasetid(ids) - 1
C     Adjust window for sat/wisdom and not-sat/wisdom data
C     SATWND, SATSND, SATRAD, and WISDOM hava a different recwin 
C     behavior for non 0 and 4 values that we want.  We adjust the 
C     behavior here.  We force values entered for 1-3 to be 4,
C     instead of being 0.
C     
C     Uncoded dataset check
         if(datasetnm(ds_i+1).eq.' ')then
             write(*,*)'Uncoded dataset ',ds_i
            GO TO 103
         endif
         if(ids.eq.9.or.ids.eq.11.or.ids.eq.12)then
            CALL MSETWIN(minbck,minfwd,satwis_recwin,istatus)
            if(istatus.ne.success_p)stop
c$$$            write(*,*)'Dataset #',ids,' MBCK',minbck,' MFWD',minfwd,
c$$$     1           ' WTYP ',satwis_recwin
         else
            CALL MSETWIN(minbck,minfwd,recwin,istatus)
            if(istatus.ne.success_p)stop
c$$$            write(*,*)'Dataset #',ids,' MBCK',minbck,' MFWD',minfwd,
c$$$     1           ' WTYP ',recwin
         endif

         CALL MDSSTA(ds_i,atime,ipvd,pvdrnam,
     1        nsta,stanam,wmoid,lat,lon,elev,timeob,provdr,istatus,
     1        source_n)
         IF(ISTATUS.NE.SUCCESS_P)then
               write(*,*)'Error reading metadata ',ds_i,istatus
            GO TO 103
         endif
C     ACARS special handling
         if(datasetnm(ds_i+1).eq.'ACARS')THEN

            CALL MGETACARS(atime,lat_vcode,nsta,nobs,lat,lat_qcd,
     1           lat_qca,lat_qcr,istatus)
            if(istatus.ne.success_p)then
               write(*,*)'Error getting acars lat ',istatus
               GO TO 103
            endif
            CALL MGETACARS(atime,lon_vcode,nsta,nobs,lon,lon_qcd,
     1           lon_qca,lon_qcr,istatus)
            if(istatus.ne.success_p)then
               write(*,*)'Error getting acars lon ',istatus
               GO TO 103
            endif
            CALL MGETACARS(atime,alt_vcode,nsta,nobs,elev,alt_qcd,
     1           alt_qca,alt_qcr,istatus)
            if(istatus.ne.success_p)then
               write(*,*)'Error getting acars elev ',istatus
               GO TO 103
            endif
            if(ORIGAPT)then
c$$$               write(*,*)'Reading acars origin airport ',istatus
               CALL MGETACARSC(atime,oairport_vcode,nsta,ORIGAIRPORT,
     1 istatus)
C Clean text, only                
               if(istatus.ne.success_p)then
                  write(*,*)'Error get acars origin airport ',istatus
                  GO TO 103
               endif
            endif
            if(DESTAPT)then
c$$$               write(*,*)'Reading acars dest airport ',istatus
               CALL MGETACARSC(atime,dairport_vcode,nsta,DESTAIRPORT,
     1              istatus)
               if(istatus.ne.success_p)then
                  write(*,*)'Error get acars dest airport ',istatus
                  GO TO 103
               endif
            endif
         endif


         IF(ISTATUS.NE.SUCCESS_P)GO TO 103
C     Profile Type
C     -------------------
C     -20       Profile data (-14)
C     0        Single-dimensioned (not a profile)
         


C ALWAYS INIT THE DATA
         
         do ii = 1,lastnobs
            do i = 1, MAXPRODS
               rdata1(ii,i) = miss_p
               qca1(ii,i) = 0
               qcr1(ii,i) = 0
               qcd1(ii,i) = 'Z'
            enddo
            i = 1
            rdata1ht(ii,i) = miss_p
            qca1ht(ii,i) = 0
            qcr1ht(ii,i) = 0
            qcd1ht(ii,i) = 'Z'
         enddo


         def_vtype = ' '
         def_profile_type = -9999
         do i = 1, nvar
            validcode(i) = .true.
            CALL MGETVCN(vcode(i),vcn,vtype(i),istatus)
            if(istatus.ne.success_p)then
               validcode(i) = .false.
            else
               CALL MGENVCN(vcn,ds_i,idb,dummy,profile_type(i))
               if(vtype(i).eq.'N')then
                  def_vtype = vtype(i)
                  def_profile_type = profile_type(i)
               else
                  if( i.ne.ORIGAPTIDX .and. i.ne.DESTAPTIDX)then
                     WRITE(*,*)'Only numeric variables are '//
     1                    'available at this time.'
                  endif
                  validcode(i) = .false.
               endif
            endif
         enddo

c$$$         WRITE(*,*)'START DS: ',datasetnm(ds_i+1),' ',def_profile_type

         if(def_profile_type.eq.-9999)go to 103


         if(def_vtype.eq.'N')then

C     Make sure the user didn't select a mismatch by mixing variables
C     with different profile or data types on the same input line.  
C     (This is against the rules in this program because we're expecting
C     the same number of levels for all variables output on the same
C     line -- one output line per level will result.)

            j = def_profile_type
            c = def_vtype
            lastnobs = 0
            do i = 1, nvar
               if(validcode(i) .and. 
     1              (profile_type(i).ne.j.or.vtype(i).ne.c))then
                  write(*,*)'Variable/profile type mismatch:?? ',i,
     1                 nvar,validcode(i),' ',j,' ',vtype(i)
                  write(*,519)'Variable/profile type mismatch: ',
     1                 (vcode(k),k=1,nvar),profile_type(i),j,vtype(i),c
 519              format(1x,a,4(1x,a),i5,i5,' ',a,' ',a)
                  go to 110
               endif
            enddo


c$$$            if(def_profile_type.ne.0 .and. nvar.gt.14)then
            if(def_profile_type.ne.0 .and. nvar.gt.MAXPRODS)then
               write(*,*)'Only MAXPRODS array variables allowed'
               nvar = MAXPRODS
            else if (nvar.gt.MAXPRODS)then
               write(*,*)'Only ',MAXPRODS,' single-dimen '//
     1              'variables allowed'
               nvar = MAXPRODS
            endif
            
            nstamax = 0


            nobs = 0
            nsta = 0
            nl = 0;
C     PROFILES
 146        format(/,1x,a,a,a,i5,a,i7)

C     GET DATA

            CALL MGETDS(DS_I,ATIME,VCODE,NVAR,VALIDCODE,PROFILE_TYPE,
     1           NSTA,NOBS,MAXDPS,MAXPRODS,NLEVELS,NLEVELSHT,
     1           NSTAMAX,NL,RDATA1,QCA1,QCR1,QCD1,RDATA1HT,
     1           QCA1HT,QCR1HT,QCD1HT,ISTATUS)

            if(NOBS.gt.MAXDPS)NOBS = MAXDPS
            if(NOBS.gt.lastnobs)lastnobs = NOBS
c$$$            write(*,*)'NL: ',nl


            csvstr = ' '
            subpvdr = '        '

C Check for hour/second in seconds product
            do k=1,nvar
C Multiple arrays check
               if(vcode(k).eq.'OBSEC')then
                  do i = 1, nstamax
                     if(rdata1(i,k).gt.0)then
                        rdata1(i,k) = mod(rdata1(i,k),60.0)
                     endif
                  enddo
               endif
            enddo

C Output stations with data
            do 200 i = 1, nstamax

C     Use the time output format desired by the user.

               if(tform.eq.0)then

                  tstr = '  '//timeob(i)
                  WRITE(*,*)'ONLY TIMESTR 1 ALLOWED'
                  CALL MTRNTIM(timeob(i),2,tstr,istatus)
               else if(tform.eq.1)then

                  CALL MTRNTIM(timeob(i),2,tstr,istatus)

               endif

               dstr = tstr(5:6)//'/'//tstr(7:8)//'/'//tstr(1:4)
               hmstr = tstr(10:11)//':'//tstr(12:13)
C     LAT LONG ELEV DATE TIME SITE PROVDR SUBPRV DATASET

C#               write(*,*)'LAT ',i,lat(i),lon(i),' ',stanam(i)

C No valid location, skip line
c$$$               if(lat(i).lt.-90.or.lat(i).gt.99.or.
c$$$     1            lon(i).le.-180.or.lon(i).ge.180  )then
               if(lat(i).lt.-90.or.lat(i).gt.90.or.
     1            lon(i).le.-180.or.lon(i).ge.180
     1              )then
c$$$                  write(*,*)'MISS ',i+1,lat(i),lon(i),
c$$$     1                 ' ',stanam(i)
                  go to 200
               endif

               if(ilatlon.eq.1)then
                  write(csvstr,1000)lnnum,elev(i),lat(i),lon(i),
     1                 dstr,hmstr,stanam(i),provdr(i),subpvdr,
     2                 datasetnm(ds_i+1)
               else
                  write(csvstr,1001)lnnum,elev(i),lat(i),lon(i),
     1                 dstr,hmstr,stanam(i),provdr(i),subpvdr,
     2                 datasetnm(ds_i+1)
               endif


 1000          format(
     3              i2,',',f8.1,',',f10.6,',',f11.6,',',
     4              a,',',a,',',a,',',a,',',a,',',a)
 1001          format(
     3              i2,',',f8.1,',',f10.2,',',f11.2,',',
     4              a,',',a,',',a,',',a,',',a,',',a)
               
CCCCC TBD WHAT ABOUT N FIELDS WITH MUTLIPLE DATA FIELDS, SEE SFCDUMP

               if(nl.le.1)then
                  if(elev(i).eq.miss_p.or.elev(i).eq.out_miss)go to 200
                  totnvar = nvar
                  do k=1,nvar
C Multiple arrays check
                     if(vcode(k).eq.'SKYCVLB')then
                        totnvar = totnvar + 5
                     else if(vcode(k).eq.'PCPINT')then
                        totnvar = totnvar + 1
                     else if(vcode(k).eq.'PCPTYPE')then
                        totnvar = totnvar + 1
                     endif
                  enddo
                  do k=1,totnvar
C     CHECK FOR DATA
                     if(rdata1(i,k).gt.-9999.and.
     1                    rdata1(i,k).lt.999999)then
C     HANDLE MISSING VALUES
                        do kk=1,totnvar
                           if(rdata1(i,kk).le.-9999.or.
     1                          rdata1(i,kk).ge.999999)then
                              if(imissval .eq. 1)then
                                 CHARRDATA1(i,kk) = ''
                              else
                                 write(CHARRDATA1(i,kk),2002)
     1                                out_miss
                              endif
C Check for origin airport
                              if(kk.eq.ORIGAPTIDX)then
C Clean non chars, only 0-9 a-Z allowed.
                                 CHARRDATA1(i,kk) = ''
                                 cnt_chars = 0
                                 do jj = 1,6
                                    ich = ichar(ORIGAIRPORT(i)(jj:jj))
c$$$                                    write(*,*)jj,ich,' ',
c$$$     1                                   ORIGAIRPORT(i)(jj:jj),' < ',
c$$$     1                                   ORIGAIRPORT(i)(jj:jj),' < '
                                    if((ich.ge.48.and.ich.le.57) .or.
     1                                   (ich.ge.65.and.ich.le.90) .or.
     1                                   (ich.ge.97.and.ich.le.122))then
                                       CHARRDATA1(i,kk)(jj:jj) = 
     1                                      ORIGAIRPORT(i)(jj:jj)
                                       cnt_chars = cnt_chars + 1
                                    else
                                       go to 5010
                                    endif
                                 enddo
 5010                            continue
                                 if(cnt_chars.eq.0)then
                                    if(imissval .eq. 1)then
                                       CHARRDATA1(i,kk) = ''
                                    else
                                       write(CHARRDATA1(i,kk),2002)
     1                                      out_miss
                                    endif
                                 endif

C Check for destination airport
                              else if(kk.eq.DESTAPTIDX)then
C Clean non chars, only 0-9 a-Z allowed.
                                 CHARRDATA1(i,kk) = ''
                                 cnt_chars = 0
                                 do jj = 1,6
                                    ich = ichar(DESTAIRPORT(i)(jj:jj))
c$$$                                    write(*,*)jj,ich,' ',
c$$$     1                                   DESTAIRPORT(i)(jj:jj),' < ',
c$$$     1                                   DESTAIRPORT(i)(jj:jj),' < '
                                    if((ich.ge.48.and.ich.le.57) .or.
     1                                   (ich.ge.65.and.ich.le.90) .or.
     1                                   (ich.ge.97.and.ich.le.122))then
                                       CHARRDATA1(i,kk)(jj:jj) = 
     1                                      DESTAIRPORT(i)(jj:jj)
                                       cnt_chars = cnt_chars + 1
                                    else
                                       go to 5011
                                    endif
                                 enddo
 5011                            continue
                                 if(cnt_chars.eq.0)then
                                    if(imissval .eq. 1)then
                                       CHARRDATA1(i,kk) = ''
                                    else
                                       write(CHARRDATA1(i,kk),2002)
     1                                      out_miss
                                    endif
                                 endif
                              endif
                           else
                              write(CHARRDATA1(i,kk),2002)
     1                             rdata1(i,kk)
                           endif
                        enddo
                        if(itextxml.eq.3)then
                           write(1,2001)csvstr,elev(i),j,(
     1                          CHARRDATA1(i,kk),qcd1(i,kk),
     1                          kk=1,totnvar)
                        else if(itextxml.eq.4)then
                           write(1,4001)csvstr,elev(i),j,(
     1                          CHARRDATA1(i,kk),qcd1(i,kk),
     2                          qca1(i,kk),qcr1(i,kk),
     3                          kk=1,totnvar)
                        else
                           write(1,3001)csvstr,elev(i),j,(
     1                          CHARRDATA1(i,kk),
     1                          kk=1,totnvar)
                        endif
c$$$
c$$$                        else
c$$$                           do kk=1,totnvar
c$$$                              if(rdata1(i,kk).eq.-9999.or.
c$$$     1                             rdata1(i,kk).eq.999999)
c$$$     1                             rdata1(i,kk) = out_miss
c$$$                           enddo
c$$$                           if(itextxml.eq.3)then
c$$$                              write(1,2000)csvstr,elev(i),j,(
c$$$     1                             rdata1(i,kk),qcd1(i,kk),kk=1,totnvar)
c$$$                           else if(itextxml.eq.4)then
c$$$                              
c$$$                              write(1,4000)csvstr,elev(i),j,(
c$$$     1                             rdata1(i,kk),qcd1(i,kk),
c$$$     2                             qca1(i,kk),qcr1(i,kk),
c$$$     3                             kk=1,totnvar)
c$$$                           else
c$$$                              write(1,3000)csvstr,elev(i),j,(
c$$$     1                             rdata1(i,kk),kk=1,totnvar)
c$$$                           endif
c$$$                        endif
                        go to 106 
                     endif
                  enddo
 106              continue
               else
                  do 700 j = 1, NLEVELS(i)
                     idx = (i-1)*nl+j
                     if(idx .LE. MAXPRODS)then
                        do k=1,nvar
C     CHECK FOR DATA AND ELEVATION
C     VALID ELEVATION METERS -990 <=> 99999 
                           if(rdata1(idx,k).gt.-9999.and.
     1                          rdata1(idx,k).lt.999999.and.
     2                          rdata1ht(idx,1).lt.99999.and.
     2                          rdata1ht(idx,1).gt.-990)then
                              dataht = rdata1ht(idx,1)
                              if(dataht.eq.-9999.or.dataht.eq.999999)
     1                             dataht = out_miss
C If elev if missing and qc is not all values, do not output
                              if(qctyp.ne.0 .and. dataht.eq.out_miss)
     1                             go to 700

C     HANDLE MISSING VALUES
                              if(imissval .eq. 1)then
                                 if(dataht.eq.out_miss)then
                                    CDATAHT = ''
                                 else
                                    write(CDATAHT,7750)dataht
                                 endif
                                 
                                 do kk=1,nvar
                                    if(rdata1(idx,kk).le.-9999.or.
     1                                   rdata1(idx,kk).ge.999999)then
                                       CHARRDATA1(idx,kk) = ''
                                    else
                                       write(CHARRDATA1(idx,kk),2002)
     1                                      rdata1(idx,kk)
                                    endif
                                 enddo
                                 if(itextxml.eq.3)then
                                    write(1,7001)csvstr,CDATAHT,j,
     1                                   (CHARRDATA1(idx,kk),
     1                                   qcd1(idx,kk),kk=1,nvar)
                                 else if(itextxml.eq.4)then
                                    write(1,9001)csvstr,CDATAHT,j,
     1                                   (CHARRDATA1(idx,kk),
     2                                   qcd1(idx,kk),qca1(idx,kk),                                    
     3                                   qcr1(idx,kk),kk=1,nvar)
                                 else
                                    write(1,8001)csvstr,CDATAHT,j,
     1                                   (CHARRDATA1(idx,kk),
     1                                   kk=1,nvar)
                                 endif
                              else
                                 do kk=1,nvar
                                    if(rdata1(idx,kk).eq.-9999.or.
     1                                   rdata1(idx,kk).eq.999999)
     1                                   rdata1(idx,kk) = out_miss
                                 enddo
                                 if(itextxml.eq.3)then
                                    write(1,2000)csvstr,dataht,j,
     1                                   (rdata1(idx,kk),qcd1(idx,kk),
     1                                   kk=1,nvar)
                                 else if(itextxml.eq.4)then
                                    
                                    write(1,4000)csvstr,
     1                                   dataht,j,
     1                                   (rdata1(idx,kk),qcd1(idx,kk),
     2                                   qca1(idx,kk),qcr1(idx,kk),
     3                                   kk=1,nvar)
                                 else
                                    write(1,3000)csvstr,dataht,j,
     1                                   (rdata1(idx,kk),
     1                                   kk=1,nvar)
                                 endif
                              endif
                              go to 105
                           endif
                        enddo
                     endif
 105                 continue

 2002                format(f13.6)
 2003                format(f8.1)
 2000                format(a,',',f8.1,',',i3,50(',',f13.6,',',a))
 2001                format(a,',',f8.1,',',i3,50(',',a,',',a))
 3000                format(a,',',f8.1,',',i3,50(',',f13.6))
 3001                format(a,',',f8.1,',',i3,50(',',a))
 4000                format(a,',',f8.1,',',i3,50(',',f13.6,',',a,',',i4,
     1                    ',',i4))
 4001                format(a,',',f8.1,',',i3,50(',',a,',',a,',',i4,',',
     1                    i4))


 7750                format(f8.1)

 7001                format(a,',',a8,',',i3,50(',',a,',',a))
 8001                format(a,',',a8,',',i3,50(',',a))
 9001                format(a,',',a8,',',i3,50(',',a,',',a,',',i4,',',
     1                    i4))




 700              continue
               endif
 200        continue
         else
            write(*,*)'No numeric data selected for dataset ',ds_i
         endif

C     Process the next line of variables.
 103  continue
      go to 110

C----------------------------------------------------------------------
C     PROGRAM EXIT
C     ------------
C     
C     Error exits.

 825  write(*,503)'CSVNUMDUMP: READ ERROR IN PARAMETER FILE AT LINE ',
     1     line
 503  format(1x,a,i4)
      stop

 850  write(*,500)'CSVNUMDUMP: PREMATURE END OF PARAMETER FILE'
 875  continue


C     Normal exit.

C-----------------------------------------------------------------------
C     OPTIONAL MADIS CLOSE ROUTINE
C     ----------------------------
C     
C     If you are embedding MADIS library calls in a larger program
C     that does additional I/O using netCDF, you may wish to close
C     all the MADIS netCDF files when you're done with the portion
C     of the program that uses MADIS, simply to avoid any possible
C     issues with using too many netCDF files.  If so, just call
C     this subroutine.  Otherwise, there's no need to call the routine.

      CALL MMADISCLOSE

 900  stop
      end

C**   ---------------------------------------------------------------------
C**   REQUIRED STANDARD FSL DISCLAIMER (NOVEMBER 2000)
C**   ---------------------------------------------------------------------
C**   OPEN SOURCE LICENSE/DISCLAIMER,  FORECAST SYSTEMS LABORATORY
C**   NOAA/OAR/FSL, 325 BROADWAY BOULDER, CO 80305
C**   
C**   THIS SOFTWARE IS DISTRIBUTED UNDER THE OPEN SOURCE DEFINITION, 
C**   WHICH MAY BE FOUND AT http://www.opensource.org/osd.html.
C**   
C**   IN PARTICULAR, REDISTRIBUTION AND USE IN SOURCE AND BINARY FORMS,
C**   WITH OR WITHOUT MODIFICATION, ARE PERMITTED PROVIDED THAT THE 
C**   FOLLOWING CONDITIONS ARE MET:
C**   
C**   - REDISTRIBUTIONS OF SOURCE CODE MUST RETAIN THIS NOTICE, THIS LIST
C**   OF CONDITIONS AND THE FOLLOWING DISCLAIMER.
C**   
C**   - REDISTRIBUTIONS IN BINARY FORM MUST PROVIDE ACCESS TO THIS 
C**   NOTICE, THIS LIST OF CONDITIONS AND THE FOLLOWING DISCLAIMER, AND
C**   THE UNDERLYING SOURCE CODE.
C**   
C**   - ALL MODIFICATIONS TO THIS SOFTWARE MUST BE CLEARLY DOCUMENTED, 
C**   AND ARE SOLELY THE RESPONSIBILITY OF THE AGENT MAKING THE 
C**   MODIFICATIONS.
C**   
C**   - IF SIGNIFICANT MODIFICATIONS OR ENHANCEMENTS ARE MADE TO THIS
C**   SOFTWARE, THE FSL SOFTWARE POLICY MANAGER 
C**   (softwaremgr.fsl@noaa.gov) BE NOTIFIED.
C**   
C**   THIS SOFTWARE AND ITS DOCUMENTATION ARE IN THE PUBLIC DOMAIN AND 
C**   ARE FURNISHED "AS IS."  THE AUTHORS, THE UNITED STATES GOVERNMENT,
C**   ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO 
C**   WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE
C**   AND DOCUMENTATION FOR ANY PURPOSE.  THEY ASSUME NO RESPONSIBILITY 
C**   (1) FOR THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO 
C**   PROVIDE TECHNICAL SUPPORT TO USERS.
C**   ---------------------------------------------------------------------
