C$$$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    MCHRTIM     CONVERT INTEGER TIME TO 9-CHARACTER TIME
C   PRGMMR: M. BARTH         ORG: FSL        DATE: 99-10-20
C
C ABSTRACT: CONVERT INTEGERS FOR YEAR, MONTH, DAY, HOUR, MINUTE INTO
C   THE 9-CHARACTER TIME FORMAT USED BY MADIS ROUTINES ('YYJJJHHMM').
C
C PROGRAM HISTORY LOG:
C   99-09-20  M. BARTH    ORIGINAL VERSION
C
C USAGE:    CALL MCHRTIM (NYEAR,NMONTH,NDAY,NHOUR,NMINUTE,CHRTIME,
C                         STATUS)
C
C   INPUT ARGUMENT LIST:
C     NYEAR    - I       2-DIGIT OR 4-DIGIT YEAR (1980-2079, OR 80-79)
C     NMONTH   - I       MONTH OF YEAR (1-12)
C     NDAY     - I       DAY OF MONTH (1-31)
C     NHOUR    - I       HOUR OF DAY (0-23)
C     NMINUTE  - I       MINUTE OF HOUR (0-59)
C
C   OUTPUT ARGUMENT LIST:
C     CHRTIME  - C       'YYJJJHHMM' WHERE,
C                        'YY'  = 2-DIGIT YEAR (80-79, FOR 1980-2079)
C                        'JJJ' = JULIAN DATE WITHIN YEAR (1-366)
C                        'HH'  = HOUR
C                        'MM'  = MINUTE
C     STATUS   - I       0 - SUCCESS, OR ERROR CODE
C
C SUBPROGRAMS CALLED:
C     UNIQUE:    MERROR, M_GTI4TM, M_MKFNAM
C
C REMARKS:
C   THE 2-DIGIT YEARS WORK BETWEEN 1980 AND 2079.  SECONDS AREN'T
C   HANDLED WITH THIS SET OF TIME ROUTINES.
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN-77 + EXTENSIONS
C   MACHINE:  UNIX
C$$$

      SUBROUTINE MCHRTIM(NYEAR,NMONTH,NDAY,NHOUR,NMINUTE,CHRTIME,
     1                   STATUS)

C     ------------------------------------------------------------------
C     INCLUDES AND COMMONS
C     ------------------------------------------------------------------
      INCLUDE 'IMPLICIT'
      INCLUDE 'MADISCON'

C     ------------------------------------------------------------------
C     ARGUMENTS
C     ------------------------------------------------------------------
      INTEGER NYEAR,NMONTH,NDAY,NHOUR,NMINUTE,STATUS
      CHARACTER*9 CHRTIME

C     ------------------------------------------------------------------
C     LOCAL
C     ------------------------------------------------------------------
      INTEGER M_GTI4TM,I4TIME

C     ------------------------------------------------------------------
C     CONVERT FROM INTEGER TIME TO I4TIME.  THEN (IF OK) CONVERT TO
C     CHARACTER TIME AND RETURN.
C     ------------------------------------------------------------------
      I4TIME = M_GTI4TM(NYEAR,NMONTH,NDAY,NHOUR,NMINUTE,0,STATUS)

      IF(STATUS.EQ.SUCCESS_P)CALL M_MKFNAM(I4TIME,CHRTIME,STATUS)

      IF(STATUS.NE.SUCCESS_P)THEN

         CALL MERROR('MCHRTIM: INVALID TIME')
         CHRTIME = '         '

      ENDIF

      RETURN
      END
C**---------------------------------------------------------------------
C**REQUIRED STANDARD FSL DISCLAIMER (NOVEMBER 2000)
C**---------------------------------------------------------------------
C**  OPEN SOURCE LICENSE/DISCLAIMER,  FORECAST SYSTEMS LABORATORY
C**  NOAA/OAR/FSL, 325 BROADWAY BOULDER, CO 80305
C**
C**  THIS SOFTWARE IS DISTRIBUTED UNDER THE OPEN SOURCE DEFINITION, 
C**  WHICH MAY BE FOUND AT http://www.opensource.org/osd.html.
C**
C**  IN PARTICULAR, REDISTRIBUTION AND USE IN SOURCE AND BINARY FORMS,
C**  WITH OR WITHOUT MODIFICATION, ARE PERMITTED PROVIDED THAT THE 
C**  FOLLOWING CONDITIONS ARE MET:
C**
C**  - REDISTRIBUTIONS OF SOURCE CODE MUST RETAIN THIS NOTICE, THIS LIST
C**    OF CONDITIONS AND THE FOLLOWING DISCLAIMER.
C**
C**  - REDISTRIBUTIONS IN BINARY FORM MUST PROVIDE ACCESS TO THIS 
C**    NOTICE, THIS LIST OF CONDITIONS AND THE FOLLOWING DISCLAIMER, AND
C**    THE UNDERLYING SOURCE CODE.
C**
C**  - ALL MODIFICATIONS TO THIS SOFTWARE MUST BE CLEARLY DOCUMENTED, 
C**    AND ARE SOLELY THE RESPONSIBILITY OF THE AGENT MAKING THE 
C**    MODIFICATIONS.
C**
C**  - IF SIGNIFICANT MODIFICATIONS OR ENHANCEMENTS ARE MADE TO THIS
C**    SOFTWARE, THE FSL SOFTWARE POLICY MANAGER 
C**    (softwaremgr.fsl@noaa.gov) BE NOTIFIED.
C**
C**  THIS SOFTWARE AND ITS DOCUMENTATION ARE IN THE PUBLIC DOMAIN AND 
C**  ARE FURNISHED "AS IS."  THE AUTHORS, THE UNITED STATES GOVERNMENT,
C**  ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO 
C**  WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE
C**  AND DOCUMENTATION FOR ANY PURPOSE.  THEY ASSUME NO RESPONSIBILITY 
C**  (1) FOR THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO 
C**  PROVIDE TECHNICAL SUPPORT TO USERS.
C**---------------------------------------------------------------------
