C$$$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    MDSSTA     GET DS STATION INFO FOR ONE TIME
C   PRGMMR: L. BENJAMIN      ORG: FSL        DATE: 09-03-23
C
C ABSTRACT: GET DATA SETS STATIONS INFORMATION FOR ONE TIME
C
C PROGRAM HISTORY LOG:
C   09-03-23  L. BENJAMIN   ORIGINAL VERSION
C   14-09-17  L. BENJAMIN   ADDED SOURCE_N RETURN
C
C     
C   INPUT ARGUMENT LIST:
C     DS_I     - I       DATA SET INDEX
C     ATIME    - C       YYJJJHHMM OF THE DATA TO BE READ
C     IPVD     - I       NUMBER OF SELECTED PROVIDERS (0 = ALL)
C     PVDRNAM  - CA      SELECTED PROVIDERS NAMES
C
C   OUTPUT ARGUMENT LIST:
C     NSTA     - I       NUMBER OF STATIONS RETURNED
C     STANAM   - CA      STATION NAMES
C     WMOID    - IA      WMO ID NUMBERS
C     LAT      - RA      LATITUDES (N)
C     LON      - RA      LONGITUDES (E)
C     ELEV     - RA      STATION ELEVATIONS (M)
C     TIMEOB   - CA      YYJJJHHMM OBSERVATION TIMES
C     PROVDR   - CA      DATA PROVIDER NAMES
C     STATUS   - I       0 - SUCCESS, OR ERROR CODE
C     SOURCE_N - I       PROVIDER CODE
C
C   INPUT FILES:
C     MADIS SFC NETCDF FILES FOR ATIME
C
C     SUBPROGRAMS CALLED:
C
C     LIBRARY: - MADISLIB:  MSETSFCPVDR, MSFCSTA, MRAOBSTA,
C     MNPNSTA, MACARSSTA, MSETMAPPVDR, MMAPSTA,
C     MSETRDMTRPVDR, MRDMTRSTA, MACARSPSTA,
C     MSETHYDROPVDR, MHYDROSTA, MSETSATWNDPVDR,
C     MSATWNDSTA, MSETSATSNDPVDR, MSATSNDSTA,
C     MSETSATRADPVDR, MSATRADSTA, MSETSNOWPVDR,
C     MSNOWSTA
C
C REMARKS:
C   - THE CALLER MUST CALL MINIT BEFORE CALLING THIS ROUTINE.
C   - THIS ROUTINE SHOULD BE CALLED WHENEVER ACCESS TO A NEW TIME
C     IS DESIRED.
C   - DATASETS THAT RETURN ELEVATION IN PRESSURE ARE CONVERTED
C     TO A HEIGHT
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN-77 + EXTENSIONS
C   MACHINE:  UNIX
C$$$
       

      SUBROUTINE MDSSTA (DS_I,ATIME,IPVD,PVDRNAM,NSTA 
     1     ,STANAM,WMOID,LAT,LON,ELEV,TIMEOB,PROVDR,STATUS,SOURCE_N)

      INCLUDE 'IMPLICIT'
      INCLUDE 'MADISCON'
      INCLUDE 'MADISCOM'
      INCLUDE 'STRCCOM'


C     ------------------------------------------------------------------
C     ARGUMENTS
C     ------------------------------------------------------------------
      INTEGER IPVD,PVDRALEN
      CHARACTER*(*) PVDRNAM(*)
      CHARACTER*9 ATIME,TIMEOB(*)
      CHARACTER*(*) PROVDR(*),STANAM(*)
      INTEGER NSTA,WMOID(*),STATUS,DS_I
      REAL LAT(*),LON(*),ELEV(*)
      REAL M_PTOZSA
      INTEGER SOURCE_N(*)

C     ------------------------------------------------------------------
C     LOCAL
C     ------------------------------------------------------------------
      REAL P(MXSTA_P)
      INTEGER I,ISTATUS
      LOGICAL PVDR_SUCCESS
C----------------------------------------------------------------------
      PVDR_SUCCESS = .FALSE.
 401  FORMAT(3X,A)
      IF(DS_I .EQ. 0)THEN
         IF(IPVD.GT.0)THEN
            CALL MSETSFCPVDR('ALL-SFC',.false.,istatus)
            DO I = 1,IPVD
               CALL MSETSFCPVDR(PVDRNAM(I),.TRUE.,ISTATUS)
               IF(ISTATUS.EQ.SUCCESS_P)THEN
                  write(*,401)PVDRNAM(I)
                  PVDR_SUCCESS = .TRUE.
               ENDIF
            ENDDO
            IF(.NOT.PVDR_SUCCESS)THEN
               CALL MSETSFCPVDR('ALL-SFC',.true.,istatus)
               write(*,401)'ALL-SFC'
               PVDR_SUCCESS = .TRUE.
            ENDIF
         ELSE
            write(*,401)'ALL-SFC'
            PVDR_SUCCESS = .TRUE.
         ENDIF
         CALL MSFCSTA(ATIME,NSTA,STANAM,WMOID,LAT,
     1        LON,ELEV,TIMEOB,PROVDR,STATUS,SOURCE_N)

      ELSE IF(DS_I .EQ. 1)THEN
         CALL MRAOBSTA(ATIME,NSTA,STANAM,WMOID,
     1        LAT,LON,ELEV,TIMEOB,STATUS)
         DO I = 1, NSTA
            PROVDR(I) = ' '
         ENDDO
         PVDR_SUCCESS = .TRUE.
      ELSE IF(DS_I .EQ. 2)THEN
         CALL MNPNSTA(ATIME,NSTA,STANAM,WMOID,LAT,
     1        LON,ELEV,TIMEOB,STATUS)
         DO I = 1, NSTA
            PROVDR(I) = ' '
         ENDDO
         PVDR_SUCCESS = .TRUE.
      ELSE IF(DS_I .EQ. 3)THEN
         CALL MACARSSTA(ATIME,NSTA,STANAM,TIMEOB,
     1        STATUS)
         DO I = 1, NSTA
            PROVDR(I) = ' '
         ENDDO
         PVDR_SUCCESS = .TRUE.
      ELSE IF(DS_I .EQ. 4)THEN
         IF(IPVD.GT.0)THEN
            CALL MSETMAPPVDR('ALL-MAP',.false.,istatus)
            DO I = 1,IPVD
               CALL MSETMAPPVDR(PVDRNAM(I),.TRUE.,ISTATUS)
               IF(ISTATUS.EQ.SUCCESS_P)THEN
                  write(*,401)PVDRNAM(I)
                  PVDR_SUCCESS = .TRUE.
               ENDIF
            ENDDO
            IF(.NOT.PVDR_SUCCESS)THEN
               CALL MSETMAPPVDR('ALL-MAP',.true.,istatus)
               write(*,401)'ALL-MAP'
               PVDR_SUCCESS = .TRUE.
            ENDIF
         ELSE
            write(*,401)'ALL-MAP'
            PVDR_SUCCESS = .TRUE.
         ENDIF
         CALL MMAPSTA(ATIME,NSTA,STANAM,WMOID,LAT,
     1        LON,ELEV,TIMEOB,PROVDR,STATUS)
      ELSE IF(DS_I .EQ. 5)THEN
         IF(IPVD.GT.0)THEN
            CALL MSETRDMTRPVDR('ALL-RDMTR',.false.,istatus)
            DO I = 1,IPVD
               CALL MSETRDMTRPVDR(PVDRNAM(I),.TRUE.,ISTATUS)
               IF(ISTATUS.EQ.SUCCESS_P)THEN
                  write(*,401)PVDRNAM(I)
                  PVDR_SUCCESS = .TRUE.
               ENDIF
            ENDDO
            IF(.NOT.PVDR_SUCCESS)THEN
               CALL MSETRDMTRPVDR('ALL-RDMTR',.true.,istatus)
               write(*,401)'ALL-RDMTR'
               PVDR_SUCCESS = .TRUE.
            ENDIF
         ELSE
            write(*,401)'ALL-RDMTR'
            PVDR_SUCCESS = .TRUE.
         ENDIF
         CALL MRDMTRSTA(ATIME,NSTA,STANAM,WMOID,
     1        LAT,LON,ELEV,TIMEOB,
     2        PROVDR,STATUS)
      ELSE IF(DS_I .EQ. 6)THEN
         CALL MACARSPSTA(ATIME,NSTA,STANAM,WMOID,LAT,LON,
     1        ELEV,TIMEOB,STATUS)
         DO I = 1, NSTA
            PROVDR(I) = ' '
         ENDDO
         PVDR_SUCCESS = .TRUE.
      ELSE IF(DS_I .EQ. 7)THEN
         IF(IPVD.GT.0)THEN
            CALL MSETHYDROPVDR('ALL-HYDRO',.false.,istatus)
            DO I = 1,IPVD
               CALL MSETHYDROPVDR(PVDRNAM(I),.TRUE.,ISTATUS)
               IF(ISTATUS.EQ.SUCCESS_P)THEN
                  write(*,401)PVDRNAM(I)
                  PVDR_SUCCESS = .TRUE.
               ENDIF
            ENDDO
            IF(.NOT.PVDR_SUCCESS)THEN
               CALL MSETHYDROPVDR('ALL-HYDRO',.true.,istatus)
               write(*,401)'ALL-HYDRO'
               PVDR_SUCCESS = .TRUE.
            ENDIF
         ELSE
            write(*,401)'ALL-HYDRO'
            PVDR_SUCCESS = .TRUE.
         ENDIF
         CALL MHYDROSTA(ATIME,NSTA,STANAM,WMOID,LAT,LON,
     1        ELEV,TIMEOB,PROVDR,STATUS)
      ELSE IF(DS_I .EQ. 8)THEN
         IF(IPVD.GT.0)THEN
            CALL MSETSATWNDPVDR('ALL-GOES-O',.false.,istatus)
            DO I = 1,IPVD
               CALL MSETSATWNDPVDR(PVDRNAM(I),.TRUE.,ISTATUS)
               IF(ISTATUS.EQ.SUCCESS_P)THEN
                  write(*,401)PVDRNAM(I)
                  PVDR_SUCCESS = .TRUE.
               ENDIF
            ENDDO
            IF(.NOT.PVDR_SUCCESS)THEN
               CALL MSETSATWNDPVDR('ALL-GOES-O',.true.,istatus)
               write(*,401)'ALL-GOES-O'
               PVDR_SUCCESS = .TRUE.
            ENDIF
         ELSE
            write(*,401)'ALL-GOES-O'
            PVDR_SUCCESS = .TRUE.
         ENDIF
         CALL MSATWNDSTA(ATIME,NSTA,PROVDR,LAT,LON,
     1        P,TIMEOB,STATUS)
         DO I = 1, NSTA
            STANAM(I) = ' '
C     ------------------------------------------------------------------
C     IN ORDER TO GET HEIGHT, WE READ IN PRESSURE (JUST DID THIS) THEN
C     CONVERT TO PRESSURE ALTITUDE USING THE U.S. STANDARD ATMOSPHERE
C     CALCULATION.  NOTE THAT THE M_PTOZSA ROUTINE EXPECTS P IN MB.
C     ------------------------------------------------------------------
            IF(P(I).NE.MISS_P)
     1           ELEV(I) = M_PTOZSA(P(I)/100.)
         ENDDO

      ELSE IF(DS_I .EQ. 10)THEN
         IF(IPVD.GT.0)THEN
            CALL MSETSATSNDPVDR('ALL-POES',.false.,istatus)
            DO I = 1,IPVD
               CALL MSETSATSNDPVDR(PVDRNAM(I),.TRUE.,ISTATUS)
               IF(ISTATUS.EQ.SUCCESS_P)THEN
                  write(*,401)PVDRNAM(I)
                  PVDR_SUCCESS = .TRUE.
               ENDIF
            ENDDO
            IF(.NOT.PVDR_SUCCESS)THEN
               CALL MSETSATSNDPVDR('ALL-POES',.true.,istatus)
               write(*,401)'ALL-POES'
               PVDR_SUCCESS = .TRUE.
            ENDIF
         ELSE
            write(*,401)'ALL-POES'
            PVDR_SUCCESS = .TRUE.
         ENDIF
         CALL MSATSNDSTA(ATIME,NSTA,PROVDR,LAT,LON,
     1        ELEV,TIMEOB,STATUS)
         DO I = 1, NSTA
            STANAM(I) = ' '
         ENDDO
      ELSE IF(DS_I .EQ. 11)THEN
         IF(IPVD.GT.0)THEN
            CALL MSETSATRADPVDR('ALL-POES',.false.,istatus)
            DO I = 1,IPVD
               CALL MSETSATRADPVDR(PVDRNAM(I),.TRUE.,ISTATUS)
               IF(ISTATUS.EQ.SUCCESS_P)THEN
                  write(*,401)PVDRNAM(I)
                  PVDR_SUCCESS = .TRUE.
               ENDIF
            ENDDO
            IF(.NOT.PVDR_SUCCESS)THEN
               CALL MSETSATRADPVDR('ALL-POES',.true.,istatus)
               write(*,401)'ALL-POES'
               PVDR_SUCCESS = .TRUE.
            ENDIF
         ELSE
            write(*,401)'ALL-POES'
            PVDR_SUCCESS = .TRUE.
         ENDIF
         CALL MSATRADSTA(ATIME,NSTA,PROVDR,LAT,LON,
     1        ELEV,TIMEOB,STATUS)
         DO I = 1, NSTA
            STANAM(I) = ' '
         ENDDO
      ELSE IF(DS_I .EQ. 12)THEN
         IF(IPVD.GT.0)THEN
            CALL MSETSNOWPVDR('ALL-SNOW',.false.,istatus)
            DO I = 1,IPVD
               CALL MSETSNOWPVDR(PVDRNAM(I),.TRUE.,ISTATUS)
               IF(ISTATUS.EQ.SUCCESS_P)THEN
                  write(*,401)PVDRNAM(I)
                  PVDR_SUCCESS = .TRUE.
               ENDIF
            ENDDO
            IF(.NOT.PVDR_SUCCESS)THEN
               CALL MSETSNOWPVDR('ALL-SNOW',.true.,istatus)
               write(*,401)'ALL-SNOW'
               PVDR_SUCCESS = .TRUE.
            ENDIF
         ELSE
            write(*,401)'ALL-SNOW'
            PVDR_SUCCESS = .TRUE.
         ENDIF
         CALL MSNOWSTA(ATIME,NSTA,STANAM,WMOID,LAT,LON,
     1        ELEV,TIMEOB,PROVDR,STATUS)
      ELSE IF(DS_I .EQ. 13)THEN
         CALL MWISDOMSTA (ATIME,NSTA,STANAM,PROVDR,LAT,LON,ELEV,
     1     TIMEOB,STATUS)
         write(*,401)'ALL-WISDOM'
         PVDR_SUCCESS = .TRUE.
      ENDIF
      IF(.NOT.PVDR_SUCCESS)THEN
         WRITE(*,401)'FAILED TO INITIALIZE DATA PROVIDER FOR DATASET'
         ISTATUS = FAILURE_P
      ENDIF
      RETURN
      END

C**   ---------------------------------------------------------------------
C**   REQUIRED STANDARD FSL DISCLAIMER (NOVEMBER 2000)
C**   ---------------------------------------------------------------------
C**   OPEN SOURCE LICENSE/DISCLAIMER,  FORECAST SYSTEMS LABORATORY
C**   NOAA/OAR/FSL, 325 BROADWAY BOULDER, CO 80305
C**   
C**   THIS SOFTWARE IS DISTRIBUTED UNDER THE OPEN SOURCE DEFINITION, 
C**   WHICH MAY BE FOUND AT http://www.opensource.org/osd.html.
C**   
C**   IN PARTICULAR, REDISTRIBUTION AND USE IN SOURCE AND BINARY FORMS,
C**   WITH OR WITHOUT MODIFICATION, ARE PERMITTED PROVIDED THAT THE 
C**   FOLLOWING CONDITIONS ARE MET:
C**   
C**   - REDISTRIBUTIONS OF SOURCE CODE MUST RETAIN THIS NOTICE, THIS LIST
C**   OF CONDITIONS AND THE FOLLOWING DISCLAIMER.
C**   
C**   - REDISTRIBUTIONS IN BINARY FORM MUST PROVIDE ACCESS TO THIS 
C**   NOTICE, THIS LIST OF CONDITIONS AND THE FOLLOWING DISCLAIMER, AND
C**   THE UNDERLYING SOURCE CODE.
C**   
C**   - ALL MODIFICATIONS TO THIS SOFTWARE MUST BE CLEARLY DOCUMENTED, 
C**   AND ARE SOLELY THE RESPONSIBILITY OF THE AGENT MAKING THE 
C**   MODIFICATIONS.
C**   
C**   - IF SIGNIFICANT MODIFICATIONS OR ENHANCEMENTS ARE MADE TO THIS
C**   SOFTWARE, THE FSL SOFTWARE POLICY MANAGER 
C**   (softwaremgr.fsl@noaa.gov) BE NOTIFIED.
C**   
C**   THIS SOFTWARE AND ITS DOCUMENTATION ARE IN THE PUBLIC DOMAIN AND 
C**   ARE FURNISHED "AS IS."  THE AUTHORS, THE UNITED STATES GOVERNMENT,
C**   ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO 
C**   WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE
C**   AND DOCUMENTATION FOR ANY PURPOSE.  THEY ASSUME NO RESPONSIBILITY 
C**   (1) FOR THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO 
C**   PROVIDE TECHNICAL SUPPORT TO USERS.
C**   ---------------------------------------------------------------------
