C$$$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    MGETDS     GET DATA SET DATA
C   PRGMMR: L. BENJAMIN      ORG: FSL        DATE: 09-03-23
C
C ABSTRACT: COMMON INTERFACE FOR FETCHING ANY DATA SET DATA.
C
C PROGRAM HISTORY LOG:
C   09-03-23  L. BENJAMIN   ORIGINAL VERSION
C     
C     
C   INPUT ARGUMENT LIST:
C     PROFILE_TYPE
C     DS_I     - I       DATA SET INDEX
C     ATIME    - C       YYJJJHHMM OF THE DATA TO BE READ
C     VCODE    - CA      VARIABLE(S) CODE NAME
C     MAXDPS   - I       MAXIMUM NUMBER OF DATA POINTS POSSIBLE
C     MAXPRODS - I       MAXIMUM NUMBER OF VARIABLES POSSIBLE
C     NVAR     - I       NUMBER OF VARIABLES TO READ
C
C   OUTPUT ARGUMENT LIST:
C     VALIDCODE - BA     VALID CODE FLAG 
C     PROFILE_TYPE -I    VARIABLES PROFILE (ARRAY) TYPE
C     NSTA     - I       NUMBER OF STATIONS RETURNED
C     NOBS     - I       NUMBER OF NON-MISSING OBS
C     NLEVELS  - IA      NUMBER OF LEVELS
C     NSTAMAX  - I       MAXIMUM NUMBER OF VARIABLES READ
C     NL       - I       VARIABLES ARRAY SIZE
C     OB       - RA      OBSERVATIONS IN M/K/S UNITS
C     QCDESC   - CA      QC DATA DESCRIPTORS
C     QCAPP    - IA      QC APPLIED WORD (SHOWS CHECKS APPLIED)
C     QCRES    - IA      QC RESULTS WORD (GOOD/BAD FOR EACH CHECK)
C     RDATA1HT - RA      OBSERVATIONS ELEVATION
C     QCA1HT   - IA      OBSERVATIONS ELEVATION QC APPLIED WORD
C     QCR1HT   - IA      OBSERVATIONS ELEVATION QC RESULTS WORD
C     QCD1HT   - CA      OBSERVATIONS ELEVATION QC DATA DESCRIPTORS
C     ISTATUS  - I       0 - SUCCESS, OR ERROR CODE
C
C   INPUT FILES:
C     MADIS MAP NETCDF FILES FOR ATIME
C
C SUBPROGRAMS CALLED:
C     UNIQUE:    MGETRAOB, MGETMAP, MGETNPN, MGETRDMTR,
C                MGETSATWND, MGETSATSND, MGETSATRAS,MGETACARS,
C                MGETSFC, MGETHYDRO, MGETSNOW,MGETACARSP
C
C   - THE CALLER MUST CALL MINIT AND MSFCSTA BEFORE CALLING THIS
C     ROUTINE.  
C   - DOUBLE & CHARACTER DATA TYPES ARE NOT HANDLED BY THIS ROUTINE.
C   - SEE madistmp.f FOR AN EXPLANATION OF HOW TEMPORARY STORAGE
C     IS HANDLED (ALL ARRAYS NAMED *TMP*).
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN-77 + EXTENSIONS
C   MACHINE:  UNIX
C$$$

      SUBROUTINE MGETDS(DS_I,ATIME,VCODE,NVAR,VALIDCODE,PROFILE_TYPE,
     1     NSTA,NOBS,MAXDPS,MAXPRODS,NLEVELS,NLEVELS2,
     1     NSTAMAX,NL,RDATA1,QCA1,QCR1,QCD1,RDATA1HT,
     1     QCA1HT,QCR1HT,QCD1HT,ISTATUS)


C     ------------------------------------------------------------------
C     INCLUDES AND COMMONS
C     ------------------------------------------------------------------
      INCLUDE 'IMPLICIT'
      INCLUDE 'MADISCON'

C     ------------------------------------------------------------------
C     ARGUMENTS
C     ------------------------------------------------------------------
      CHARACTER*9 ATIME
      INTEGER NSTA,NOBS,STATUS,DS_I,NVAR,NL,NSTAMAX,PROFILE_TYPE(*)
      CHARACTER*(*) VCODE(*)
      LOGICAL VALIDCODE(*)
      INTEGER MAXDPS,MAXPRODS
      integer NLEVELS(*)
      integer NLEVELS2(*)
      REAL RDATA1(MAXDPS,MAXPRODS)
      INTEGER QCA1(MAXDPS,MAXPRODS),QCR1(MAXDPS,MAXPRODS)
      CHARACTER*1 QCD1(MAXDPS,MAXPRODS)
      REAL RDATA1HT(MAXDPS,MAXPRODS)
      INTEGER QCA1HT(MAXDPS,MAXPRODS),QCR1HT(MAXDPS,MAXPRODS)
      CHARACTER*1 QCD1HT(MAXDPS,MAXPRODS)

C     ------------------------------------------------------------------
C     LOCAL
C     ------------------------------------------------------------------
      INTEGER I,ISTATUS,II,INVAR,J
      INTEGER CHANNEL,IDX

C----------------------------------------------------------------------
C     DEFAULTS
      NL = 1

C----------------------------------------------------------------------
C     RAOB
      IF(DS_I .EQ. 1)THEN

         IF(PROFILE_TYPE(1).EQ.0)THEN
            NL = 405
         ELSE IF(PROFILE_TYPE(1).EQ.2)THEN
            NL = 22
         ELSE IF(PROFILE_TYPE(1).EQ.3)THEN
            NL = 175
         ELSE IF(PROFILE_TYPE(1).EQ.4)THEN
            NL = 200
         ELSE IF(PROFILE_TYPE(1).EQ.5)THEN
            NL = 4
         ELSE IF(PROFILE_TYPE(1).EQ.6)THEN
            NL = 4
         ELSE 
            NL = 1
         ENDIF
         
         DO I = 1, NVAR
            IF(VALIDCODE(I))THEN
               CALL MGETRAOB(ATIME,VCODE(I),NSTA,NOBS,NLEVELS,
     1           RDATA1(1,I),QCD1(1,I),QCA1(1,I),QCR1(1,I),
     2           ISTATUS)
               IF(ISTATUS.EQ.SUCCESS_P)THEN
                  IF(NSTA.GT.NSTAMAX)NSTAMAX = NSTA
                  IF(PROFILE_TYPE(1).EQ.0)THEN
                     CALL MGETRAOB(ATIME,'HT     ',NSTA,NOBS,
     1                    NLEVELS2,RDATA1HT(1,I),QCD1HT(1,I),
     2                    QCA1HT(1,I),QCR1HT(1,I),
     3                    ISTATUS) 
                  ELSE IF(PROFILE_TYPE(1).EQ.2)THEN
                     CALL MGETRAOB(ATIME,'HTMAN  ',NSTA,NOBS,
     1                    NLEVELS2,RDATA1HT(1,I),QCD1HT(1,I),
     2                    QCA1HT(1,I),QCR1HT(1,I),
     3                    ISTATUS) 
                  ELSE IF(PROFILE_TYPE(1).EQ.3)THEN
                     CALL MGETRAOB(ATIME,'HTSIG  ',NSTA,NOBS,
     1                    NLEVELS2,RDATA1HT(1,I),QCD1HT(1,I),
     2                    QCA1HT(1,I),QCR1HT(1,I),
     3                    ISTATUS) 
                  ELSE IF(PROFILE_TYPE(1).EQ.4)THEN
                     CALL MGETRAOB(ATIME,'HTWIND ',NSTA,NOBS,
     1                    NLEVELS2,RDATA1HT(1,I),QCD1HT(1,I),
     2                    QCA1HT(1,I),QCR1HT(1,I),
     3                    ISTATUS) 
                  ELSE IF(PROFILE_TYPE(1).EQ.5)THEN
                     CALL MGETRAOB(ATIME,'HTMXWND',NSTA,NOBS,
     1                    NLEVELS2,RDATA1HT(1,I),QCD1HT(1,I),
     2                    QCA1HT(1,I),QCR1HT(1,I),
     3                    ISTATUS) 
                  ELSE IF(PROFILE_TYPE(1).EQ.6)THEN
                     CALL MGETRAOB(ATIME,'HTTROP ',NSTA,NOBS,
     1                    NLEVELS2,RDATA1HT(1,I),QCD1HT(1,I),
     2                    QCA1HT(1,I),QCR1HT(1,I),
     3                    ISTATUS) 
                  ENDIF
               ELSE
                  WRITE(*,*)"FAILED: ",VCODE(I)
               ENDIF
            ENDIF
         ENDDO


C----------------------------------------------------------------------
C     MAP
      ELSE IF(DS_I .EQ. 4)THEN

         IF(PROFILE_TYPE(1).EQ.-10)THEN
            NL = 202
         ELSE IF(PROFILE_TYPE(1).EQ.0)THEN
            NL = 1
         ELSE IF(PROFILE_TYPE(1).LE.-11.AND.
     1           PROFILE_TYPE(1).GE.-13)THEN
            NL = 50
         ELSE IF(PROFILE_TYPE(1).EQ.-14)THEN
            NL = 30
         ELSE 
            NL = 5
         ENDIF
         
         DO I = 1, NVAR
            CALL MGETMAP(ATIME,VCODE(I),NSTA,NOBS,NLEVELS,
     1           RDATA1(1,I),QCD1(1,I),QCA1(1,I),QCR1(1,I),
     2           ISTATUS)
            IF(ISTATUS.EQ.SUCCESS_P.AND.NSTA.GT.NSTAMAX)THEN
               NSTAMAX = NSTA
            ENDIF
         ENDDO
         IF(NSTAMAX.GT.0)THEN
            I = 1
            IF(PROFILE_TYPE(1).EQ.-10)THEN
               CALL MGETMAP(ATIME,'HT',NSTA,NOBS,
     1              NLEVELS2,RDATA1HT(1,I),QCD1HT(1,I),
     2              QCA1HT(1,I),QCR1HT(1,I),
     3              ISTATUS) 
               IF(ISTATUS.NE.SUCCESS_P)THEN
                  WRITE(*,*),'ERROR FETCHING HT ',ISTATUS
               ENDIF
            ELSE IF(PROFILE_TYPE(1).EQ.-14)THEN
               CALL MGETMAP(ATIME,'HTR1',NSTA,NOBS,
     1              NLEVELS2,RDATA1HT(1,I),QCD1HT(1,I),
     2              QCA1HT(1,I),QCR1HT(1,I),
     3              ISTATUS) 
               IF(ISTATUS.NE.SUCCESS_P)THEN
                  WRITE(*,*),'ERROR FETCHING HT ',ISTATUS
               ENDIF
               
            ELSE IF(PROFILE_TYPE(1).EQ.-11)THEN
               CALL MGETMAP(ATIME,'HTW1',NSTA,NOBS,
     1              NLEVELS2,RDATA1HT(1,I),QCD1HT(1,I),
     2              QCA1HT(1,I),QCR1HT(1,I),
     3              ISTATUS) 
               IF(ISTATUS.NE.SUCCESS_P)THEN
                  WRITE(*,*),'ERROR FETCHING HT ',ISTATUS
               ENDIF
            ELSE IF(PROFILE_TYPE(1).EQ.-12)THEN
               CALL MGETMAP(ATIME,'HTW2',NSTA,NOBS,
     1              NLEVELS2,RDATA1HT(1,I),QCD1HT(1,I),
     2              QCA1HT(1,I),QCR1HT(1,I),
     3              ISTATUS) 
               IF(ISTATUS.NE.SUCCESS_P)THEN
                  WRITE(*,*),'ERROR FETCHING HT ',ISTATUS
               ENDIF
            ELSE IF(PROFILE_TYPE(1).EQ.-13)THEN
               CALL MGETMAP(ATIME,'HTW3',NSTA,NOBS,
     1              NLEVELS2,RDATA1HT(1,I),QCD1HT(1,I),
     2              QCA1HT(1,I),QCR1HT(1,I),
     3              ISTATUS) 
               IF(ISTATUS.NE.SUCCESS_P)THEN
                  WRITE(*,*),'ERROR FETCHING HT ',ISTATUS
               ENDIF
            ENDIF

         ENDIF


C----------------------------------------------------------------------
C     ARRAYS

      ELSE IF(PROFILE_TYPE(1).LE.-1)THEN
C     THE MGET ROUTINE IS USED TO READ ALL NUMERIC VARIABLES.
C     ALL VARIABLES ARE RETURNED AS FORTRAN REAL TYPE , THE
C     QC DATA DESCRIPTORS ARE ALWAYS CHARACTER*1 , AND THE
C     BIT MASKS SHOWING THE QC CHECKS THAT WERE APPLIED AND
C     THEIR RESULTS  ARE ALWAYS INTEGER.
         IF(DS_I .EQ. 0)THEN
            WRITE(*,*)'TBD, RAOB moved, should not be here.'
c$$$         ELSE IF(DS_I .EQ. 1)THEN
c$$$            IF(PROFILE_TYPE(1).EQ.0)THEN
c$$$               NL = 405
c$$$            ELSE IF(PROFILE_TYPE(1).EQ.2)THEN
c$$$               NL = 22
c$$$            ELSE IF(PROFILE_TYPE(1).GE.3)THEN
c$$$               NL = 175
c$$$            ELSE IF(PROFILE_TYPE(1).EQ.4)THEN
c$$$               NL = 200
c$$$            ELSE IF(PROFILE_TYPE(1).EQ.5)THEN
c$$$               NL = 4
c$$$            ELSE IF(PROFILE_TYPE(1).EQ.6)THEN
c$$$               NL = 4
c$$$            ELSE 
c$$$               NL = 1
c$$$            ENDIF
c$$$            
c$$$            DO I = 1, NVAR
c$$$               IF(VALIDCODE(I))THEN
c$$$                  CALL MGETRAOB(ATIME,VCODE(I),NSTA,NOBS,NLEVELS,
c$$$     1                 RDATA1(1,I),QCD1(1,I),QCA1(1,I),QCR1(1,I),
c$$$     2                 ISTATUS)
c$$$                  IF(ISTATUS.EQ.SUCCESS_P)THEN
c$$$                     IF(NSTA.GT.NSTAMAX)NSTAMAX = NSTA
c$$$                     CALL MGETRAOB(ATIME,'HT',NSTA,NOBS,
c$$$     1                    NLEVELS,RDATA1HT(1,I),QCD1HT(1,I),
c$$$     2                    QCA1HT(1,I),QCR1HT(1,I),
c$$$     3                    ISTATUS) 
c$$$
c$$$                  ELSE
c$$$                     WRITE(*,*)"FAILED: ",VCODE(I)
c$$$                  ENDIF
c$$$               ENDIF
c$$$            ENDDO
            
         ELSE IF(DS_I .EQ. 2)THEN
            IF(PROFILE_TYPE(1).EQ.-2.OR.PROFILE_TYPE(1).EQ.-3)THEN
               NL = 36
            ELSE 
               NL = 64
            ENDIF
            DO I = 1, NVAR
               IF(VALIDCODE(I))THEN
                  CALL MGETNPN(ATIME,VCODE(I),NSTA,NOBS,NLEVELS,
     1                 RDATA1(1,I),QCD1(1,I),QCA1(1,I),QCR1(1,I),
     2                 ISTATUS)
c$$$                  write(*,*)'NPN ', istatus,nobs
                  IF(ISTATUS.EQ.SUCCESS_P)THEN
                     IF(NSTA.GT.NSTAMAX)NSTAMAX = NSTA
                     IF(PROFILE_TYPE(1).EQ.-2)THEN
                     CALL MGETNPN(ATIME,'HTL',NSTA,NOBS,
     1                    NLEVELS2,RDATA1HT(1,I),QCD1HT(1,I),
     2                    QCA1HT(1,I),QCR1HT(1,I),
     3                    ISTATUS) 
c$$$                        NL = 36
                     ELSE IF(PROFILE_TYPE(1).EQ.-3)THEN
                     CALL MGETNPN(ATIME,'HTH',NSTA,NOBS,
     1                    NLEVELS2,RDATA1HT(1,I),QCD1HT(1,I),
     2                    QCA1HT(1,I),QCR1HT(1,I),
     3                    ISTATUS) 
c$$$                        NL = 36
                     ELSE 
                     CALL MGETNPN(ATIME,'HT',NSTA,NOBS,
     1                    NLEVELS2,RDATA1HT(1,I),QCD1HT(1,I),
     2                    QCA1HT(1,I),QCR1HT(1,I),
     3                    ISTATUS) 
c$$$                        NL = 64
                     ENDIF
c$$$                  write(*,*)'NPN=ht ', istatus,nobs
                  ENDIF
               ENDIF
            ENDDO                     
         ELSE IF(DS_I .EQ. 3)THEN
         ELSE IF(DS_I .EQ. 4)THEN
         ELSE IF(DS_I .EQ. 5)THEN
            NL = 100
            DO I = 1, NVAR
               IF(VALIDCODE(I))THEN
                  CALL MGETRDMTR(ATIME,VCODE(I),NSTA,NOBS,NLEVELS,
     1                 RDATA1(1,I),QCD1(1,I),QCA1(1,I),QCR1(1,I),
     2                 ISTATUS)
                  IF(ISTATUS.EQ.SUCCESS_P)THEN
                     IF(NSTA.GT.NSTAMAX)NSTAMAX = NSTA
                     CALL MGETRDMTR(ATIME,'HT',NSTA,NOBS,
     1                    NLEVELS2,RDATA1HT(1,I),QCD1HT(1,I),
     2                    QCA1HT(1,I),QCR1HT(1,I),
     3                    ISTATUS) 
                  ENDIF
               ENDIF
            ENDDO
         ELSE IF(DS_I .EQ. 6)THEN
            NL = 200
            DO I = 1, NVAR
               IF(VALIDCODE(I))THEN
                  CALL MGETACARSP(ATIME,VCODE(I),NSTA,NOBS,NLEVELS,
     1                 RDATA1(1,I),QCD1(1,I),QCA1(1,I),QCR1(1,I),
     2                 ISTATUS)
                  IF(ISTATUS.EQ.SUCCESS_P)THEN
                     IF(NSTA.GT.NSTAMAX)NSTAMAX = NSTA
                     CALL MGETACARSP(ATIME,'HT',NSTA,NOBS,NLEVELS2,
     1                    RDATA1HT(1,I),QCD1HT(1,I),
     2                    QCA1HT(1,I),QCR1HT(1,I),
     3                    ISTATUS) 
                  ENDIF
               ENDIF
            ENDDO
         ELSE IF(DS_I .EQ. 7)THEN
         ELSE IF(DS_I .EQ. 8)THEN
         ELSE IF(DS_I .EQ. 10)THEN
            NL = 40
            DO I = 1, NVAR
               IF(VALIDCODE(I))THEN
                  CALL MGETSATSND(ATIME,VCODE(I),NSTA,NOBS,NLEVELS,
     1                 RDATA1(1,I),QCD1(1,I),QCA1(1,I),QCR1(1,I),
     2                 ISTATUS)
                  IF(ISTATUS.EQ.SUCCESS_P)THEN
                     IF(NSTA.GT.NSTAMAX)NSTAMAX = NSTA
                     CALL MGETSATSND(ATIME,'HT',NSTA,NOBS,
     1                    NLEVELS2,RDATA1HT(1,I),QCD1HT(1,I),
     2                    QCA1HT(1,I),QCR1HT(1,I),
     3                    ISTATUS) 
                  ENDIF
               ENDIF
            ENDDO
         ELSE IF(DS_I .EQ. 11)THEN
            
            IF(-20 .eq. PROFILE_TYPE(1))then
               I = 1
               IF(VALIDCODE(I))THEN
C     CONVERT [STATIONS*CHANNELS,1] TO [STATIONS,CHANNEL]
                  CALL MGETSATRAD(ATIME,VCODE(I),NSTA,NOBS,
     1                 NLEVELS,RDATA1HT(1,I),QCD1HT(1,I),
     2                 QCA1HT(1,I),QCR1HT(1,I),
     3                 ISTATUS) 
                  IF(ISTATUS.EQ.SUCCESS_P)THEN                     
                     IF(NSTA.GT.MAXPRODS)NSTA=MAXPRODS
                     IF(NSTA.GT.NSTAMAX)NSTAMAX = NSTA
                     DO I = 1, NVAR
                        READ(VCODE(I)(3:4),1002)CHANNEL
 1002                   FORMAT(I2)
                        IF(CHANNEL.GE.1 .OR.CHANNEL.LE.35)THEN
                           DO II = 1, NSTA
                              IDX = (II-1)*35+CHANNEL
                              RDATA1(II,I) = RDATA1HT(IDX,1)
                              QCD1(II,I) = QCD1HT(IDX,1)
                              QCA1(II,I) = QCA1HT(IDX,1)
                              QCR1(II,I) = QCR1HT(IDX,1)
                           ENDDO
                        ENDIF
                     ENDDO
                  ENDIF
               ENDIF
            ELSE
               CALL MGETSATRAD(ATIME,VCODE(I),NSTA,NOBS,NLEVELS,
     1              RDATA1(1,I),QCD1(1,I),QCA1(1,I),QCR1(1,I),
     2              ISTATUS)
               IF(ISTATUS.EQ.SUCCESS_P.AND.NSTA.GT.NSTAMAX)
     1              NSTAMAX = NSTA
            ENDIF
c$$$            DO I = 1, NVAR
c$$$               IF(VALIDCODE(I))THEN
c$$$                  CALL MGETSATRAD(ATIME,VCODE(I),NSTA,NOBS,NLEVELS,
c$$$     1                 RDATA1(1,I),QCD1(1,I),QCA1(1,I),QCR1(1,I),
c$$$     2                 ISTATUS)
c$$$                  IF(ISTATUS.EQ.SUCCESS_P)THEN
c$$$                     IF(NSTA.GT.NSTAMAX)NSTAMAX = NSTA
c$$$                     CALL MGETSATRAD(ATIME,'HT',NSTA,NOBS,
c$$$     1                    NLEVELS,RDATA1HT(1,I),QCD1HT(1,I),
c$$$     2                    QCA1HT(1,I),QCR1HT(1,I),
c$$$     3                    ISTATUS) 
c$$$                  ENDIF
c$$$               ENDIF
c$$$            ENDDO
         ELSE IF(DS_I .EQ. 12)THEN
         ENDIF
      ELSE IF(PROFILE_TYPE(1).EQ.0)THEN
C----------------------------------------------------------------------
C     SINGLE LOCATIONS
         IF(DS_I .EQ. 0)THEN
            INVAR = 0
            DO I = 1, NVAR
               INVAR = INVAR + 1

               IF(VALIDCODE(I))THEN
                  IF(VCODE(I).NE.'SKYCVLB'.AND.VCODE(I).NE.'PCPINT'.AND.
     1                 VCODE(I).NE.'PCPTYPE')THEN
                     CALL MGETSFC(ATIME,VCODE(I),NSTA,NOBS,
     1                    RDATA1(1,INVAR),QCD1(1,INVAR),QCA1(1,INVAR),
     2                    QCR1(1,INVAR),ISTATUS)
                     
                     IF(ISTATUS.EQ.SUCCESS_P)THEN
                        IF(NSTA.GT.NSTAMAX)NSTAMAX = NSTA
                     ELSE
                        WRITE(*,*)"FAILED: ",VCODE(I)
                     ENDIF
                  ELSE IF(VCODE(I).EQ.'SKYCVLB')THEN
                     CALL MGETSFC(ATIME,VCODE(I),NSTA,NOBS,
     1                    RDATA1HT(1,INVAR),QCD1HT(1,INVAR),
     2                    QCA1HT(1,INVAR),QCR1HT(1,INVAR),ISTATUS)
C Convert 6*DPS,1 to DPS,6                     
                     DO II = 1, NSTA
                        DO J = 0,5
                           RDATA1(II,INVAR+J) = 
     1                          RDATA1HT((II-1)*6+J+1,INVAR)
                           QCD1(II,INVAR+J) = 
     1                          QCD1HT((II-1)*6+J+1,INVAR)
                           QCA1(II,INVAR+J) = 
     1                          QCA1HT((II-1)*6+J+1,INVAR)
                           QCR1(II,INVAR+J) = 
     1                          QCR1HT((II-1)*6+J+1,INVAR)
                        ENDDO
                     ENDDO
                     INVAR = INVAR + 5
                  ELSE IF(VCODE(I).EQ.'PCPINT'.OR.
     1                    VCODE(I).EQ.'PCPTYPE')THEN
                     CALL MGETSFC(ATIME,VCODE(I),NSTA,NOBS,
     1                    RDATA1HT(1,INVAR),QCD1HT(1,INVAR),
     2                    QCA1HT(1,INVAR),QCR1HT(1,INVAR),ISTATUS)
C Convert 2*DPS,1 to DPS,2
                     DO II = 1, NSTA
                        DO J = 0,1
                           RDATA1(II,INVAR+J) = 
     1                          RDATA1HT((II-1)*2+J+1,INVAR)
                           QCD1(II,INVAR+J) = 
     1                          QCD1HT((II-1)*2+J+1,INVAR)
                           QCA1(II,INVAR+J) = 
     1                          QCA1HT((II-1)*2+J+1,INVAR)
                           QCR1(II,INVAR+J) = 
     1                          QCR1HT((II-1)*2+J+1,INVAR)
                        ENDDO
                     ENDDO
                     INVAR = INVAR + 1
                  ENDIF
               ENDIF
            ENDDO
         ELSE IF(DS_I .EQ. 1)THEN
            WRITE(*,*)'TBD-2, RAOB moved, should not be here.'
c$$$            DO I = 1, NVAR
c$$$               IF(VALIDCODE(I))THEN
c$$$                  CALL MGETRAOB(ATIME,VCODE(I),NSTA,NOBS,NLEVELS,
c$$$     1                 RDATA1(1,I),QCD1(1,I),QCA1(1,I),QCR1(1,I),
c$$$     2                 ISTATUS)
c$$$                  CALL MGETRAOB(ATIME,'HT',NSTA,NOBS,NLEVELS,
c$$$     1                 RDATA1(1,I),QCD1(1,I),QCA1(1,I),QCR1(1,I),
c$$$     2                 ISTATUS)
c$$$
c$$$                  IF(ISTATUS.EQ.SUCCESS_P)THEN
c$$$                     IF(NSTA.GT.NSTAMAX)NSTAMAX = NSTA
c$$$                  ELSE
c$$$                     WRITE(*,*)"FAILED: ",VCODE(I)
c$$$                  ENDIF
c$$$               ENDIF
c$$$            ENDDO
         ELSE IF(DS_I .EQ. 2)THEN
            DO I = 1, NVAR
               IF(VALIDCODE(I))THEN
                  CALL MGETNPN(ATIME,VCODE(I),NSTA,NOBS,NLEVELS,
     1                 RDATA1(1,I),QCD1(1,I),
     2                 QCA1(1,I),QCR1(1,I),ISTATUS)
                  IF(ISTATUS.EQ.SUCCESS_P.AND.NSTA.GT.NSTAMAX)
     1                 NSTAMAX = NSTA
               ENDIF
            ENDDO
         ELSE IF(DS_I .EQ. 3)THEN
            DO I = 1, NVAR
               IF(VALIDCODE(I))THEN
                  CALL MGETACARS(ATIME,VCODE(I),NSTA,NOBS,
     1                 RDATA1(1,I),QCD1(1,I),
     2                 QCA1(1,I),QCR1(1,I),ISTATUS)
                  IF(ISTATUS.EQ.SUCCESS_P.AND.NSTA.GT.NSTAMAX)
     1                 NSTAMAX = NSTA
               ENDIF
            ENDDO
         ELSE IF(DS_I .EQ. 4)THEN
         ELSE IF(DS_I .EQ. 5)THEN
            DO I = 1, NVAR
               IF(VALIDCODE(I))THEN
                  CALL MGETRDMTR(ATIME,VCODE(I),NSTA,NOBS,NLEVELS,
     1                 RDATA1(1,I),QCD1(1,I),
     2                 QCA1(1,I),QCR1(1,I),ISTATUS)
                  IF(ISTATUS.EQ.SUCCESS_P.AND.NSTA.GT.NSTAMAX)
     1                 NSTAMAX = NSTA
               ENDIF
            ENDDO
         ELSE IF(DS_I .EQ. 6)THEN
            DO I = 1, NVAR
               IF(VALIDCODE(I))THEN
                  CALL MGETACARSP(ATIME,VCODE(I),NSTA,NOBS,
     1                 NLEVELS,RDATA1(1,I),QCD1(1,I),
     2                 QCA1(1,I),QCR1(1,I),ISTATUS)
                  IF(ISTATUS.EQ.SUCCESS_P.AND.NSTA.GT.NSTAMAX)
     1                 NSTAMAX = NSTA
               ENDIF
            ENDDO
         ELSE IF(DS_I .EQ. 7)THEN
            DO I = 1, NVAR
               IF(VALIDCODE(I))THEN
                  CALL MGETHYDRO(ATIME,VCODE(I),NSTA,NOBS,
     1                 RDATA1(1,I),QCD1(1,I),
     2                 QCA1(1,I),QCR1(1,I),ISTATUS)
                  IF(ISTATUS.EQ.SUCCESS_P.AND.NSTA.GT.NSTAMAX)
     1                 NSTAMAX = NSTA
               ENDIF
            ENDDO
         ELSE IF(DS_I .EQ. 8)THEN
            DO I = 1, NVAR
               IF(VALIDCODE(I))THEN
                  CALL MGETSATWND(ATIME,VCODE(I),NSTA,NOBS,
     1                 RDATA1(1,I),QCD1(1,I),QCA1(1,I),
     2                 QCR1(1,I),ISTATUS)
                  IF(ISTATUS.EQ.SUCCESS_P.AND.NSTA.GT.NSTAMAX)
     1                 NSTAMAX = NSTA
               ENDIF
            ENDDO
         ELSE IF(DS_I .EQ. 10)THEN
            DO I = 1, NVAR
               IF(VALIDCODE(I))THEN
                  CALL MGETSATSND(ATIME,VCODE(I),NSTA,NOBS,
     1                 NLEVELS,RDATA1(1,I),QCD1(1,I),QCA1(1,I),
     2                 QCR1(1,I),ISTATUS)
                  IF(ISTATUS.EQ.SUCCESS_P.AND.NSTA.GT.NSTAMAX)
     1                 NSTAMAX = NSTA
               ENDIF
            ENDDO
         ELSE IF(DS_I .EQ. 11)THEN
            DO I = 1, NVAR
               IF(VALIDCODE(I))THEN
                  CALL MGETSATRAD(ATIME,VCODE(I),NSTA,NOBS,
     1                 NLEVELS,RDATA1(1,I),QCD1(1,I),QCA1(1,I),
     2                 QCR1(1,I),ISTATUS)
                  IF(ISTATUS.EQ.SUCCESS_P.AND.NSTA.GT.NSTAMAX)
     1                 NSTAMAX = NSTA
               ENDIF
            ENDDO
         ELSE IF(DS_I .EQ. 12)THEN
            DO I = 1, NVAR
               IF(VALIDCODE(I))THEN
                  CALL MGETSNOW(ATIME,VCODE(I),NSTA,NOBS,
     1                 RDATA1(1,I),QCD1(1,I),QCA1(1,I),
     2                 QCR1(1,I),ISTATUS)
                  IF(ISTATUS.EQ.SUCCESS_P.AND.NSTA.GT.NSTAMAX)
     1                 NSTAMAX = NSTA
               ENDIF
            ENDDO
         ELSE IF(DS_I .EQ. 13)THEN
            DO I = 1, NVAR
               IF(VALIDCODE(I))THEN
                  CALL MGETWISDOM(ATIME,VCODE(I),NSTA,NOBS,
     1                 RDATA1(1,I),QCD1(1,I),QCA1(1,I),
     2                 QCR1(1,I),ISTATUS)
                  IF(ISTATUS.EQ.SUCCESS_P)THEN
                     IF(NSTA.GT.NSTAMAX)NSTAMAX = NSTA
                  ELSE
                     WRITE(*,*)"FAILED: ",VCODE(I)
                  ENDIF
               ENDIF
            ENDDO
         ENDIF
      ENDIF
      RETURN
      END

C**   ---------------------------------------------------------------------
C**   REQUIRED STANDARD FSL DISCLAIMER (NOVEMBER 2000)
C**   ---------------------------------------------------------------------
C**   OPEN SOURCE LICENSE/DISCLAIMER,  FORECAST SYSTEMS LABORATORY
C**   NOAA/OAR/FSL, 325 BROADWAY BOULDER, CO 80305
C**   
C**   THIS SOFTWARE IS DISTRIBUTED UNDER THE OPEN SOURCE DEFINITION, 
C**   WHICH MAY BE FOUND AT HTTP://WWW.OPENSOURCE.ORG/OSD.HTML.
C**   
C**   IN PARTICULAR, REDISTRIBUTION AND USE IN SOURCE AND BINARY FORMS,
C**   WITH OR WITHOUT MODIFICATION, ARE PERMITTED PROVIDED THAT THE 
C**   FOLLOWING CONDITIONS ARE MET:
C**   
C**   - REDISTRIBUTIONS OF SOURCE CODE MUST RETAIN THIS NOTICE, THIS LIST
C**   OF CONDITIONS AND THE FOLLOWING DISCLAIMER.
C**   
C**   - REDISTRIBUTIONS IN BINARY FORM MUST PROVIDE ACCESS TO THIS 
C**   NOTICE, THIS LIST OF CONDITIONS AND THE FOLLOWING DISCLAIMER, AND
C**   THE UNDERLYING SOURCE CODE.
C**   
C**   - ALL MODIFICATIONS TO THIS SOFTWARE MUST BE CLEARLY DOCUMENTED, 
C**   AND ARE SOLELY THE RESPONSIBILITY OF THE AGENT MAKING THE 
C**   MODIFICATIONS.
C**   
C**   - IF SIGNIFICANT MODIFICATIONS OR ENHANCEMENTS ARE MADE TO THIS
C**   SOFTWARE, THE FSL SOFTWARE POLICY MANAGER 
C**   (SOFTWAREMGR.FSL@NOAA.GOV) BE NOTIFIED.
C**   
C**   THIS SOFTWARE AND ITS DOCUMENTATION ARE IN THE PUBLIC DOMAIN AND 
C**   ARE FURNISHED "AS IS."  THE AUTHORS, THE UNITED STATES GOVERNMENT,
C**   ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO 
C**   WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE
C**   AND DOCUMENTATION FOR ANY PURPOSE.  THEY ASSUME NO RESPONSIBILITY 
C**   (1) FOR THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO 
C**   PROVIDE TECHNICAL SUPPORT TO USERS.
C**   ---------------------------------------------------------------------
