C$$$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    MGTNUM      DRIVER TO READ A NUMERIC NETCDF VARIABLE
C   PRGMMR: M. BARTH         ORG: FSL        DATE: 01-05-03
C
C ABSTRACT: DRIVER TO READ A NUMERIC NETCDF VARIABLE ON AN OPEN NETCDF
C   FILE.
C
C PROGRAM HISTORY LOG:
C   01-05-03  M. BARTH    ORIGINAL VERSION
C   08-06-13  L. BENJAMIN MOVED CONVERTING AWIPS POES SOUNDING TO 40 
C                         LEVELS FROM 50 TO THIS ROUTINE.  ALSO, 
C                         CHANGED CALL TO MRDNUM1 AND ADDED PASSING
C                         OF THE TEMPORARY PROCESSING MEMORY.

C
C  RDNUM TO RDNUM1
C
C USAGE:    CALL MGTNUM (VRNAM,NCID,RECDIM,STARTR,NRECS,SPVAL,
C                        VDATA,STATUS)
C
C   INPUT ARGUMENT LIST:
C     VRNAM    - C       NETCDF VARIABLE NAME
C     NCID     - I       ID OF THE OPEN NETCDF QC FILE
C     RECDIM   - I       DIMENSION ID OF THE FILE'S RECORD DIMENSION
C     STARTR   - I       STARTING RECORD TO READ
C     NRECS    - I       NUMBER OF RECORDS TO READ
C     SPVAL    - R       DESIRED MISSING DATA FLAG
C
C   OUTPUT ARGUMENT LIST:
C     VDATA    - RA      REAL ARRAY WITH DATA AND SPVALS FILLED IN
C     STATUS   - I       SUCCESS_P OR ERROR CODE
C
C   INPUT FILES:
C     WHATEVER NETCDF FILE'S OPEN ON NCID.
C
C SUBPROGRAMS CALLED:
C     UNIQUE:    MRAOBNUM, MRDNUM1, M_LSTCHR
C
C REMARKS:
C   SEE REMARKS IN THE SUBROUTINES CALLED BY THIS ROUTINE.
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN-77 + EXTENSIONS
C   MACHINE:  UNIX
C$$$

      SUBROUTINE MGTNUM(VRNAM,NCID,RECDIM,STARTR,NRECS,SPVAL,VDATA,
     1                  STATUS)

C     ------------------------------------------------------------------
C     INCLUDES AND COMMONS
C     ------------------------------------------------------------------
      INCLUDE 'IMPLICIT'
      INCLUDE 'MADISCON'
      INCLUDE 'MADISCOM'
      INCLUDE 'MADISTMP'

C     ------------------------------------------------------------------
C     ARGUMENTS
C     ------------------------------------------------------------------
      CHARACTER*(*) VRNAM
      INTEGER NCID,RECDIM,STARTR,NRECS,STATUS
      REAL SPVAL,VDATA(*)
      INTEGER NX
      INTEGER I,J,L,K,M_LSTCHR
      INTEGER I40,I50

      REAL RDATA(MXDATA_P)
      INTEGER IDATA(MXDATA_P)
      INTEGER*2 I2DATA(MXDATA_P)
      BYTE BDATA(MXDATA_P)

      EQUIVALENCE(RDATA,IDATA,I2DATA,BDATA,TMP_L0_N1)


C     ------------------------------------------------------------------
C     THIS IS ONLY A DRIVER ROUTINE.  CALL THE APPROPRIATE SUBROUTINE
C     FOR THE CURRENT DATASET TYPE.
C     ------------------------------------------------------------------
      IF(CDTYPE.EQ.RAOB_DS_P)THEN

         CALL MRAOBNUM(VRNAM,NCID,RECDIM,STARTR,NRECS,SPVAL,VDATA,
     1                 STATUS)

      ELSE
         
         CALL MRDNUM1(VRNAM,NCID,RECDIM,STARTR,NRECS,SPVAL,VDATA,
     1        MXDATA_P,RDATA,IDATA,I2DATA,BDATA,NX,STATUS)
         

C     ------------------------------------------------------------------
C     CONVERT AWIPS POES SOUNDING TO 40 LEVELS FROM 50.  THERE IS ONLY 
C     DATA IN THE FIRST 40 LEVELS.
C     ------------------------------------------------------------------
         IF(CDBTYPE.EQ.AWIPS_DB_P .AND. CDTYPE.EQ.SATS_DS_P)THEN
            J = LEN(VRNAM)
            L = M_LSTCHR(VRNAM,J)
            IF(VRNAM(1:L).EQ.'mixingRatio'.OR.
     1           VRNAM(1:L).EQ.'temperature'.OR.
     2           VRNAM(1:L).EQ.'pressure')THEN
               I40 = 0
               DO I = 1, NRECS
                  I50 = (I-1)*50
                  DO K = 1,40
                     I40 = I40 + 1
                     I50 = I50 + 1
                     VDATA(I40) = VDATA(I50)
                  ENDDO
               ENDDO
               DO I = I40+1, NX
                  VDATA(I) = SPVAL
               ENDDO
               
            ENDIF
         ENDIF

      ENDIF

      RETURN
      END
C**---------------------------------------------------------------------
C**REQUIRED STANDARD FSL DISCLAIMER (NOVEMBER 2000)
C**---------------------------------------------------------------------
C**  OPEN SOURCE LICENSE/DISCLAIMER,  FORECAST SYSTEMS LABORATORY
C**  NOAA/OAR/FSL, 325 BROADWAY BOULDER, CO 80305
C**
C**  THIS SOFTWARE IS DISTRIBUTED UNDER THE OPEN SOURCE DEFINITION, 
C**  WHICH MAY BE FOUND AT http://www.opensource.org/osd.html.
C**
C**  IN PARTICULAR, REDISTRIBUTION AND USE IN SOURCE AND BINARY FORMS,
C**  WITH OR WITHOUT MODIFICATION, ARE PERMITTED PROVIDED THAT THE 
C**  FOLLOWING CONDITIONS ARE MET:
C**
C**  - REDISTRIBUTIONS OF SOURCE CODE MUST RETAIN THIS NOTICE, THIS LIST
C**    OF CONDITIONS AND THE FOLLOWING DISCLAIMER.
C**
C**  - REDISTRIBUTIONS IN BINARY FORM MUST PROVIDE ACCESS TO THIS 
C**    NOTICE, THIS LIST OF CONDITIONS AND THE FOLLOWING DISCLAIMER, AND
C**    THE UNDERLYING SOURCE CODE.
C**
C**  - ALL MODIFICATIONS TO THIS SOFTWARE MUST BE CLEARLY DOCUMENTED, 
C**    AND ARE SOLELY THE RESPONSIBILITY OF THE AGENT MAKING THE 
C**    MODIFICATIONS.
C**
C**  - IF SIGNIFICANT MODIFICATIONS OR ENHANCEMENTS ARE MADE TO THIS
C**    SOFTWARE, THE FSL SOFTWARE POLICY MANAGER 
C**    (softwaremgr.fsl@noaa.gov) BE NOTIFIED.
C**
C**  THIS SOFTWARE AND ITS DOCUMENTATION ARE IN THE PUBLIC DOMAIN AND 
C**  ARE FURNISHED "AS IS."  THE AUTHORS, THE UNITED STATES GOVERNMENT,
C**  ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO 
C**  WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE
C**  AND DOCUMENTATION FOR ANY PURPOSE.  THEY ASSUME NO RESPONSIBILITY 
C**  (1) FOR THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO 
C**  PROVIDE TECHNICAL SUPPORT TO USERS.
C**---------------------------------------------------------------------
