C$$$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    MMAPLT      GET LEVEL TYPES FOR MAP INTEGRATED PROFILES
C   PRGMMR: M. BARTH         ORG: FSL        DATE: 02-05-01
C
C ABSTRACT: GET THE LEVEL TYPES USED FOR EACH MAP INTEGRATED PROFILE
C   GIVEN THE CURRENT DATA
C
C PROGRAM HISTORY LOG:
C   02-05-01  M. BARTH    ORIGINAL VERSION
C
C USAGE:    CALL MMAPLT (VCN,DSN,FNUM,NV,OB,QCDESC,QCAPP,QCRES,STATUS)
C
C   INPUT ARGUMENT LIST:
C     VCN      - I       VARIABLE CODE NUMBER
C     DSN      - I       DATASET CODE NUMBER
C     FNUM     - I       FILE NUMBER (INDEX IN COMMON) OF NETCDF FILE
C     NV       - I       NUMBER OF LEVELS
C
C   OUTPUT ARGUMENT LIST:
C     OB       - RA      LEVEL TYPE CODES (+N = WIND MODE N, 4 = RASS)
C     QCDESC   - CA      QC DATA DESCRIPTORS  -- SET TO NO QC
C     QCAPP    - RA      QC APPLIED WORD      -- SET TO NO QC
C     QCRES    - RA      QC RESULTS WORD      -- SET TO NO QC
C     STATUS   - I       SUCCESS_P OR ERROR CODE
C
C   INPUT FILES:
C     NETCDF FILE OPENED USING DATA STRUCTURES FOR FNUM IN STRCCOM
C
C SUBPROGRAMS CALLED:
C     UNIQUE:    MGTNUM
C
C REMARKS:
C   - THIS ROUTINE IS ASSUMED TO BE CALLED BY A HIGHER-LEVEL MADIS
C     ROUTINE, SO NO ERROR CHECKING IS DONE ON THE INPUT VARIABLES.
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN-77 + EXTENSIONS
C   MACHINE:  UNIX
C$$$

      SUBROUTINE MMAPLT (VCN,DSN,FNUM,NV,OB,QCDESC,QCAPP,QCRES,STATUS)

C     ------------------------------------------------------------------
C     INCLUDES AND COMMONS
C     ------------------------------------------------------------------
      INCLUDE 'IMPLICIT'
      INCLUDE 'MADISCON'
      INCLUDE 'MADISCOM'
      INCLUDE 'STRCCOM'

C     ------------------------------------------------------------------
C     ARGUMENTS
C     ------------------------------------------------------------------
      INTEGER VCN,DSN,FNUM,NV,STATUS
      REAL OB(NV,*),QCAPP(NV,*),QCRES(NV,*)
      CHARACTER*1 QCDESC(NV,*)

C     ------------------------------------------------------------------
C     LOCAL
C     ------------------------------------------------------------------
      INTEGER I,J,CD

C     ------------------------------------------------------------------
C     IF THIS ROUTINE HASN'T YET BEEN CALLED FOR THIS TIME, READ IN
C     THE LEVEL TYPES INTO MADISCOM.
C     ------------------------------------------------------------------
      STATUS = SUCCESS_P

      IF(.NOT.HAVE_MAP_LT)THEN

         CD = DSN - MAX_SFCDSET_P

         CALL MGTNUM('windRASSMode',UAPT_NCID(FNUM,CD),
     1               UAPT_RECDIM(FNUM,CD),1,UAPT_NRECS(FNUM,CD),MISS_P,
     2               MAP_LT,STATUS)
         IF(STATUS.NE.SUCCESS_P)RETURN

         HAVE_MAP_LT = .TRUE.

      ENDIF

C     ------------------------------------------------------------------
C     IF WE'VE BEEN CALLED TO RETURN THE LEVEL TYPES TO THE USER, DO SO.
C     ------------------------------------------------------------------
      IF(VCN.EQ.MAPLT_VN_P)THEN

         CD = DSN - MAX_SFCDSET_P

         DO 5 J = 1, UAPT_NRECS(FNUM,CD)

            DO 6 I = 1, MXMAPLEV_P

               OB(I,J) = MAP_LT(I,J)
               QCDESC(I,J) = 'Z'
               QCAPP(I,J) = 0
               QCRES(I,J) = 0

 6          CONTINUE

 5       CONTINUE

      ENDIF

      RETURN
      END

C**---------------------------------------------------------------------
C**REQUIRED STANDARD FSL DISCLAIMER (NOVEMBER 2000)
C**---------------------------------------------------------------------
C**  OPEN SOURCE LICENSE/DISCLAIMER,  FORECAST SYSTEMS LABORATORY
C**  NOAA/OAR/FSL, 325 BROADWAY BOULDER, CO 80305
C**
C**  THIS SOFTWARE IS DISTRIBUTED UNDER THE OPEN SOURCE DEFINITION, 
C**  WHICH MAY BE FOUND AT http://www.opensource.org/osd.html.
C**
C**  IN PARTICULAR, REDISTRIBUTION AND USE IN SOURCE AND BINARY FORMS,
C**  WITH OR WITHOUT MODIFICATION, ARE PERMITTED PROVIDED THAT THE 
C**  FOLLOWING CONDITIONS ARE MET:
C**
C**  - REDISTRIBUTIONS OF SOURCE CODE MUST RETAIN THIS NOTICE, THIS LIST
C**    OF CONDITIONS AND THE FOLLOWING DISCLAIMER.
C**
C**  - REDISTRIBUTIONS IN BINARY FORM MUST PROVIDE ACCESS TO THIS 
C**    NOTICE, THIS LIST OF CONDITIONS AND THE FOLLOWING DISCLAIMER, AND
C**    THE UNDERLYING SOURCE CODE.
C**
C**  - ALL MODIFICATIONS TO THIS SOFTWARE MUST BE CLEARLY DOCUMENTED, 
C**    AND ARE SOLELY THE RESPONSIBILITY OF THE AGENT MAKING THE 
C**    MODIFICATIONS.
C**
C**  - IF SIGNIFICANT MODIFICATIONS OR ENHANCEMENTS ARE MADE TO THIS
C**    SOFTWARE, THE FSL SOFTWARE POLICY MANAGER 
C**    (softwaremgr.fsl@noaa.gov) BE NOTIFIED.
C**
C**  THIS SOFTWARE AND ITS DOCUMENTATION ARE IN THE PUBLIC DOMAIN AND 
C**  ARE FURNISHED "AS IS."  THE AUTHORS, THE UNITED STATES GOVERNMENT,
C**  ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO 
C**  WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE
C**  AND DOCUMENTATION FOR ANY PURPOSE.  THEY ASSUME NO RESPONSIBILITY 
C**  (1) FOR THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO 
C**  PROVIDE TECHNICAL SUPPORT TO USERS.
C**---------------------------------------------------------------------
